#include <iostream>
#include <fstream>	// XXX
#include <sstream>	// XXX
#include <list>
#include <map>
#include <set>
#include <string>
#include <vector>
#include <ext/hash_map>	// XXX
#include <cassert>
#include <cstring>

#include <getopt.h>	// XXX
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <signal.h>

#include <unistd.h>
#include <sys/types.h>	// XXX
#include <sys/time.h>	// XXX
#include <fcntl.h>	// XXX
#include <sys/wait.h>	// XXX
#include <netinet/in.h>	// XXX
#include <netdb.h>	// XXX
#include <sys/socket.h>	// XXX
#include <arpa/inet.h>	// XXX

#include <xercesc/dom/deprecated/DOM_DOMException.hpp>
#include <xercesc/dom/deprecated/DOM.hpp>
#include <xercesc/dom/deprecated/DOM_Node.hpp>
#include <xercesc/dom/deprecated/DOMParser.hpp>
#include <xercesc/dom/deprecated/DOMString.hpp>
#include <xercesc/framework/MemBufInputSource.hpp>
#include <xercesc/framework/XMLFormatter.hpp>
#include <xercesc/sax/ErrorHandler.hpp>
#include <xercesc/sax/SAXParseException.hpp>
#include <xercesc/util/PlatformUtils.hpp>
#include <xercesc/util/TranscodingException.hpp>
#include <xercesc/util/XercesDefs.hpp>
#include <xercesc/util/XMLString.hpp>
#include <xercesc/util/XMLUniDefs.hpp>

using XERCES_CPP_NAMESPACE_QUALIFIER chLF;
using XERCES_CPP_NAMESPACE_QUALIFIER DOM_Attr;
using XERCES_CPP_NAMESPACE_QUALIFIER DOM_Document;
using XERCES_CPP_NAMESPACE_QUALIFIER DOM_DOMException;
using XERCES_CPP_NAMESPACE_QUALIFIER DOM_Element;
using XERCES_CPP_NAMESPACE_QUALIFIER DOM_NamedNodeMap;
using XERCES_CPP_NAMESPACE_QUALIFIER DOM_Node;
using XERCES_CPP_NAMESPACE_QUALIFIER DOMParser;
using XERCES_CPP_NAMESPACE_QUALIFIER DOMString;
using XERCES_CPP_NAMESPACE_QUALIFIER DOM_XMLDecl;
using XERCES_CPP_NAMESPACE_QUALIFIER ErrorHandler;
using XERCES_CPP_NAMESPACE_QUALIFIER MemBufInputSource;
using XERCES_CPP_NAMESPACE_QUALIFIER SAXParseException;
using XERCES_CPP_NAMESPACE_QUALIFIER XMLException;
using XERCES_CPP_NAMESPACE_QUALIFIER XMLFormatTarget;
using XERCES_CPP_NAMESPACE_QUALIFIER XMLFormatter;
using XERCES_CPP_NAMESPACE_QUALIFIER XMLPlatformUtils;
using XERCES_CPP_NAMESPACE_QUALIFIER XMLString;

using namespace std;

typedef set<string> StringSet;

/*
	../src/libmdlgen/noreach.cpp
*/

extern "C"	int	reachable(const char* topo, const char* spec);
extern "C"	void	reach_init(char** argv);

/* PDS: StateInfo.hpp */

// to make inserting into a multimap less painful
typedef multimap<string,int>::value_type mType;

typedef map<string,int> Valuation;

// class to store info for initial state for a generic FSM
class StateInfo
{
public:
  // number of variables in initial state
  int numVars;

  // number of variables to ignore, if any
  int numIVars;

  // mappings for initial state variables
  multimap<string,int> vars;

  // names of "variables" we can ignore
  set<string> ignore;
};


// structure to store information about the network
class NetInfo : public StateInfo
{
public:
  int numHosts;
  int numAttacks;
  int numServices;
  
  // mapping from names to numbers
  map<string,int> hosts;
  map<string,int> attacks;
  /* services: name -> id * port */
  map<string,pair<int,int> > services;
};

//extern NetInfo net;

// utility function to output a NetInfo struct
ostream &operator<<( ostream &out, NetInfo &net );

// parse function for NetInfo struct
void ParseNetInfo( istream &in, NetInfo &net );

// function to convert an integer to a string
char *itoa( int n, char *storage );

// utility to convert user privileges to integers
int priv2int( DOMString priv );

/* PDS: precond.hpp */

typedef int HostID;
typedef unsigned int ServiceID;
typedef unsigned int AttackID;
typedef unsigned int PortID;
typedef unsigned int FirewallID;

// HostID 'GENERAL' means the attack in question is a general attack,
// with no specific target
#define GENERAL -1

ostream& operator<< (ostream& target, const DOMString& s);
struct less_str : public binary_function <DOMString, DOMString, bool> 
{
  bool operator()( const DOMString &x, const DOMString &y) const { return x.compareString(y) < 0; }
};

typedef set <DOMString, less_str> DOMStringSet;
typedef map <DOMString, DOMString, less_str> DOMStringMap;

typedef map<DOMString, DOMStringMap, less_str> HostObjectMap;
typedef map<DOMString, DOMStringSet, less_str> HostObjectAttrNameMap;
typedef map< DOMString, PortID, less_str > NamePortIdMap;
typedef set<HostID> HostIdSet;

class ModelMemUsage
{
  static const long NodeImplSize;
  static const long ChildImplSize;
  static const long ParentImplSize;
  static const long ElementImplSize;
  static const long CharDataImplSize;
  static const long AttrImplSize;
  static const long EntityImplSize;
  static const long NotationImplSize;
  static const long DocImplSize;
  static const long DocTypeImplSize;

public:
  virtual ~ModelMemUsage() {}
  virtual long GetMemSize() const = 0;
    
  static long GetDOMStringSizeEx( const DOMString &str ) {
    return 2 * sizeof(int) + str.length() * sizeof(XMLCh);
  }
  static long GetDOMStringSize( const DOMString &str ) {
    return 0; //all strings are calculated in GetDOM_DocumentSize();
  }
  static long GetDOMStringSetSize( const DOMStringSet &str_set );
  static long GetDOMStringMapSize( const DOMStringMap &str_map );
  static long GetDOM_DocumentSize( DOM_Node *node );
  
private:
  static long GetDOM_NodeSize( DOM_Node *node, DOMStringSet &used_str );
};

struct FirewallRule {
  int srchost;
  int dsthost;
  int srcport;
  int dstport;
    
  bool operator == ( const FirewallRule &rule ) const {
    return (rule.srchost == srchost &&
            rule.dsthost == dsthost &&
            rule.dstport == dstport &&
            rule.srcport == srcport); 
  }
  
  bool less ( const FirewallRule &rule ) const {
    bool res = (srchost < rule.srchost);
    
    if ( !res && srchost == rule.srchost)
    {
      res = (dsthost < rule.dsthost);
      if ( !res && dsthost == rule.dsthost )
        if ( !(res = (srcport < rule.srcport)) && srcport == rule.srcport )
          res = (dstport < rule.dstport);
    }
    
    return res;
  }
  
  // returns 1 if 'this' is equal to 'rule'
  // returns -1 if 'rule' includes 'this'
  // returns 2 if 'this' include 'rule'
  // return 0 otherwise
  int MatchWildacrdRule( const FirewallRule &rule ) const {
    int res = 0;
    int srchostres = 0;
    int srcportres = 0;
    int dsthostres = 0;
    int dstportres = 0;
    
    if ( rule.srchost == srchost )
      srchostres = 1;
    else if ( rule.srchost == -1 )
      srchostres = -1;
    else if ( srchost == -1 )
      srchostres = 2;
    
    if ( rule.srcport == srcport )
      srcportres = 1;
    else if ( rule.srcport == -1 )
      srcportres = -1;
    else if ( srcport == -1 )
      srcportres = 2;
    
    if ( rule.dsthost == dsthost )
      dsthostres = 1;
    else if ( rule.dsthost == -1 )
      dsthostres = -1;
    else if ( dsthost == -1 )
      dsthostres = 2;
    
    if ( rule.dstport == dstport )
      dstportres = 1;
    else if ( rule.dstport == -1 )
      dstportres = -1;
    else if ( dstport == -1 )
      dstportres = 2;
    
    if ( srchostres == srcportres && srchostres == dstportres &&
         srchostres == dsthostres)
      res = srchostres;
    else if ( abs(srchostres) == 1 && abs(srcportres) == 1 && 
              abs(dsthostres) == 1 && abs(dstportres) == 1 )
      res = -1;
    else if ( srchostres > 0 && srcportres > 0 && dsthostres > 0 && dstportres > 0 )
      res = 2;
    
    return res;    
  }
  
  static string GetPortConstantName( const DOMString &port ) {
    return string( "PORT_" ) + string( port.transcode() );
  }
  
};

class Precondition: public ModelMemUsage {

public:
  enum PreconditionType {
    PRIVILEGE,
    SERVICE_PORT_AGREEMENT,
    VULNERABILITY,
    SERVICE,
    CONNECTIVITY,
    TRUST,
    NOTRUST,
    KNOWLEDGE,
    OBJECT,
    SENTOBJECT,
    FIREWALLRULE,
    FIREWALLREACHABLE,
    AND,
    OR,
    NOT,
    UNDEFINED
  };

  Precondition() {
    type = UNDEFINED;
  }
  Precondition(PreconditionType tp) { type = tp; }
  virtual ~Precondition() {}

  virtual void Print(ostream& out) const {};
  virtual void PrintSPIN(ostream& out) const {};

protected:
  static char *RelToString(DOMString rel);

private:

  PreconditionType type;
};

typedef list<Precondition *> ListPrecond;

class PrivilegePrecondition : public Precondition {

public:

  PrivilegePrecondition() : Precondition(PRIVILEGE) {}
  PrivilegePrecondition(DOMString& h, char *r, DOMString& v) : Precondition(PRIVILEGE) {
    host = h;
    rel = r;
    value = v;
  }
  virtual ~PrivilegePrecondition() { }

  virtual void Print(ostream& out) const;
  virtual void PrintSPIN(ostream& out) const;
  
  long GetMemSize() const {
    return GetDOMStringSize( host ) + strlen( rel ) + 
           GetDOMStringSize( value ) + sizeof(PrivilegePrecondition);
  }
  
private:
  DOMString host;
  char *rel;
  DOMString value;
};

class SPAPrecondition : public Precondition {

public:

  SPAPrecondition() : Precondition(SERVICE_PORT_AGREEMENT) {}
  SPAPrecondition(DOMString& s) : Precondition(SERVICE_PORT_AGREEMENT) { service = s; }
  virtual ~SPAPrecondition() {}

  virtual void Print(ostream& out) const;
  virtual void PrintSPIN(ostream& out) const;
    
  long GetMemSize() const {
    return GetDOMStringSize( service ) + sizeof(SPAPrecondition);
  }
  
private:
  DOMString service;
};

class ServicePrecondition : public Precondition {

public:

  ServicePrecondition() : Precondition(SERVICE) {}
  ServicePrecondition(DOMString& h, DOMString& n) : Precondition(SERVICE) {
    host = h;
    name = n;
  }
  virtual ~ServicePrecondition() {}

  virtual void Print(ostream& out) const;
  virtual void PrintSPIN(ostream& out) const;
    
  long GetMemSize() const {
    return GetDOMStringSize( host ) + GetDOMStringSize( name ) + sizeof(ServicePrecondition);
  }

private:
  DOMString host;
  DOMString name;
};

class Network;

class ConnectivityPrecondition : public Precondition {

public:

  ConnectivityPrecondition( Network *net ) : Precondition(CONNECTIVITY) { network = net; }
  ConnectivityPrecondition( Network *net, DOMString& s, ServiceID i, int connsrc ) : Precondition(CONNECTIVITY) {
    network = net;
    service = s;
    id = i;
    src = connsrc;
  }
  virtual ~ConnectivityPrecondition() {}

  virtual void Print(ostream& out) const;
  virtual void PrintSPIN(ostream& out) const;
    
  long GetMemSize() const {
    return GetDOMStringSize( service ) + sizeof(ConnectivityPrecondition);
  }

private:
  Network *network;
  DOMString service;
  ServiceID id;
  int src;
};

class TrustPrecondition : public Precondition {

public:

  TrustPrecondition() : Precondition(TRUST) {}
  virtual ~TrustPrecondition() {}

  virtual void Print(ostream& out) const;
  virtual void PrintSPIN(ostream& out) const;
    
  long GetMemSize() const {
    return sizeof(TrustPrecondition);
  }

private:
};

class NoTrustPrecondition : public Precondition {

public:

  NoTrustPrecondition( Network *net ) : Precondition(NOTRUST) { network = net; }
  virtual ~NoTrustPrecondition() {}

  virtual void Print(ostream& out) const;
  virtual void PrintSPIN(ostream& out) const;
    
  long GetMemSize() const {
    return sizeof(NoTrustPrecondition);
  }

private:
  Network *network;
};

class VulnerabilityPrecondition : public Precondition {

public:

  VulnerabilityPrecondition() : Precondition(VULNERABILITY) {}
  VulnerabilityPrecondition(DOMString& h, DOMString& n) : Precondition(VULNERABILITY) {
    host = h;
    name = n;
  }
  virtual ~VulnerabilityPrecondition() {}

  virtual void Print(ostream& out) const ;
  virtual void PrintSPIN(ostream& out) const;
    
  long GetMemSize() const {
    return GetDOMStringSize( host ) + GetDOMStringSize( name ) + sizeof(VulnerabilityPrecondition);
  }

private:
  DOMString host;
  DOMString name;
};

class KnowledgePrecondition : public Precondition {

public:

  KnowledgePrecondition() : Precondition(KNOWLEDGE) {}
  KnowledgePrecondition(DOMString& n, DOMString& v) : Precondition(KNOWLEDGE), name(n), value(v) {}
  virtual ~KnowledgePrecondition();

  virtual void Print(ostream& out) const ;
  virtual void PrintSPIN(ostream& out) const;
    
  long GetMemSize() const {
    return GetDOMStringSize( name ) + GetDOMStringSize( value ) + sizeof(KnowledgePrecondition);
  }

private:
  DOMString name;
  DOMString value;
};

class ObjectPrecondition : public Precondition {

public:

  ObjectPrecondition() : Precondition(OBJECT) {}
  ObjectPrecondition(int host_name, const DOMString& obj_name) : Precondition(OBJECT) { 
    m_host = host_name; m_obj_name = obj_name; 
  }
  virtual ~ObjectPrecondition() {}

  virtual void Print(ostream& out) const;
  virtual void PrintSPIN(ostream& out) const;
  
  void   SetObjAttribute( const DOMString &attr_name, const DOMString &attr_value ) {
      m_obj_attributes[attr_name] = attr_value;
  }
  
  void SetHost( int host ) { m_host = host; }
  int GetHost( ) { return m_host; }
  
  long GetMemSize() const {
    return GetDOMStringSize( m_obj_name ) + GetDOMStringMapSize( m_obj_attributes ) + 
           sizeof(ObjectPrecondition);
  }
  
private:
  DOMString m_obj_name;
  int m_host;
  DOMStringMap m_obj_attributes;
};

class SentObjectPrecondition : public Precondition {

public:

  SentObjectPrecondition() : Precondition(SENTOBJECT) { 
    m_set_source = false;
    m_srchost = 0; 
    m_dsthost = 0;
  }
  
  SentObjectPrecondition(int dsthost, const DOMString& service, const DOMString& obj_name) : Precondition(SENTOBJECT) { 
    m_service = service; 
    m_dsthost = dsthost;
    m_obj_name = obj_name; 
    m_set_source = false;
  }
  virtual ~SentObjectPrecondition() {}

  virtual void Print(ostream& out) const;
  virtual void PrintSPIN(ostream& out) const;
  
  void SetSource( int host ) { m_srchost = host; m_set_source = true; }
  
  long GetMemSize() const {
    return GetDOMStringSize( m_obj_name ) + GetDOMStringSize( m_service ) + 
           sizeof(SentObjectPrecondition);
  }
  
private:
  DOMString m_obj_name;
  int m_srchost;
  int m_dsthost;
  DOMString m_service; 
  bool m_set_source;
  //DOMStringMap m_obj_attributes;
};

class FirewallRulePrecondition: public Precondition {
  
public:
  FirewallRulePrecondition(): Precondition(FIREWALLRULE) { }
  FirewallRulePrecondition( Network *net, HostID srchost, PortID srcport, 
                            HostID dsthost, PortID dstport, bool result = true ): Precondition(FIREWALLRULE) { 
    m_network = net;
    m_srchost = srchost;
    m_dsthost = dsthost;
    m_srcport = srcport;
    m_dstport = dstport;
    m_result  = result;
  }
  ~FirewallRulePrecondition(){}
    
  virtual void Print(ostream& out) const;
  virtual void PrintSPIN(ostream& out) const;
  
  long GetMemSize() const {
    return sizeof( FirewallRulePrecondition );
  }
  
private:
  Network  *m_network;
  HostID    m_srchost;
  HostID    m_dsthost;
  PortID    m_srcport;
  PortID    m_dstport;
  bool      m_result;
};

class FirewallReachablePrecondition: public Precondition {
  
public:
  FirewallReachablePrecondition( int srchost, const DOMString &srcport, 
                                 int dsthost, const DOMString &dstport): Precondition(FIREWALLREACHABLE) {
    m_srchost = srchost;
    m_dsthost = dsthost;
    m_srcport = srcport;
    m_dstport = dstport;
  }
  ~FirewallReachablePrecondition(){}
    
  virtual void Print(ostream& out) const;
  virtual void PrintSPIN(ostream& out) const;
  
  long GetMemSize() const {
    return sizeof( FirewallReachablePrecondition ) +
           GetDOMStringSize( m_srcport ) + GetDOMStringSize( m_dstport );
  }
    
private:
  int    m_srchost;
  int    m_dsthost;
  DOMString m_srcport;
  DOMString m_dstport;
};

class OrPrecondition : public Precondition {

public:

  OrPrecondition() : Precondition(OR) {}
  virtual ~OrPrecondition();

  virtual void AddDisjunct(Precondition *child) { children.push_back(child); }

  virtual void Print(ostream& out) const;
  virtual void PrintSPIN(ostream& out) const;
  
  long GetMemSize() const {
    return children.size() * sizeof(Precondition *) + sizeof(OrPrecondition);
  }

private:
  ListPrecond children;
};

class AndPrecondition : public Precondition {

public:

  AndPrecondition() : Precondition(AND) {}
  virtual ~AndPrecondition();

  virtual void AddConjunct(Precondition *child) { children.push_back(child); }

  virtual void Print(ostream& out) const;
  virtual void PrintSPIN(ostream& out) const;
  
  long GetMemSize() const {
    return children.size() * sizeof(Precondition *) + sizeof(AndPrecondition);
  }

private:
  ListPrecond children;
};

class NotPrecondition : public Precondition {

public:

  NotPrecondition() : Precondition(NOT), cond(NULL) {}
  NotPrecondition(Precondition *p) : Precondition(NOT), cond(p) {}
  virtual ~NotPrecondition();

  virtual void Print(ostream& out) const;
  virtual void PrintSPIN(ostream& out) const;
    
  long GetMemSize() const {
    return sizeof(NotPrecondition);
  }

private:
  Precondition *cond;
};

class Effect: public ModelMemUsage {

public:

  enum EffectType {
    PRIVILEGE,
    SERVICE,
    VULNERABILITY,
    CONNECTIVITY,
    TRUST,
    KNOWLEDGE,
    SENDOBJECT,
    PASSARG,
    FIREWALLACTION,
    UNDEFINED
  };

  Effect() { type = UNDEFINED; }
  Effect( const DOMString& h, const DOMString& s, const DOMString& n, const DOMString& v, AttackID aid, EffectType t) {
    host = h;
    source = s;
    name = n;
    value = v;
    attackid = aid;
    type = t;
  }
  virtual ~Effect() {  }

  DOMString GetName() { return name; }

  virtual void Print(ostream& out) const;
  virtual void PrintSPIN(ostream& out) const;
  
  virtual long GetMemSize() const {
    return GetDOMStringSize( host ) + GetDOMStringSize( source ) +
           GetDOMStringSize( name ) + GetDOMStringSize( value ) + sizeof(Effect);
  }
  
public:
  DOMString host;
  DOMString source;
  DOMString name;
  DOMString value;
  AttackID  attackid;
  EffectType type;
};

class SendObjectEffect: public Effect {
  
public:
  SendObjectEffect() { type = Effect::SENDOBJECT;}
  SendObjectEffect( const DOMString& srchost, const DOMString& dsthost, const DOMString& dstsvc, const DOMString& objname, AttackID aid ): 
                   Effect( dsthost, srchost, objname, objname, aid, Effect::SENDOBJECT) 
  {
    m_service = dstsvc;
  };
  ~SendObjectEffect() {}
  virtual void Print(ostream& out) const;
  virtual void PrintSPIN(ostream& out) const;
  DOMString GetSrcHost() const { return source; }
  DOMString GetDstHost() const { return host; }
  DOMString GetDstService() const { return m_service; } 
  DOMString GetObjName() const { return value; } 
  void SetObjName( const DOMString &name ) { value = name; }
  void SetDstHost( const DOMString &name ) { host = name; }
  void SetDstService( const DOMString &name ) { m_service = name; }
  void SetSrcHost( const DOMString &name ) { source = name; }
  
  long GetMemSize() const {
    return Effect::GetMemSize() + GetDOMStringSize( m_service ) + 
           sizeof(SendObjectEffect) - sizeof(Effect); 
  }
  
private:
  DOMString m_service;
};

class PassArgEffect: public Effect {
  
public:
  PassArgEffect() { type = Effect::PASSARG;}
  PassArgEffect( const DOMString& service, const DOMString& objname, AttackID aid ): 
                   Effect( DOMString::transcode(""), service, objname, objname, aid, Effect::PASSARG) 
  {};
  ~PassArgEffect() {}
  virtual void Print(ostream& out) const;
  virtual void PrintSPIN(ostream& out) const;
  DOMString GetService() const { return source; }
  DOMString GetObjName() const { return value; } 
  void SetObjName( const DOMString &name ) { value = name; }
  void SetService( const DOMString &name ) { source = name; }
};

class FirewallActionEffect: public Effect {

public:
  FirewallActionEffect( Network *net, bool status, AttackID aid ):
                       Effect( DOMString::transcode(""), DOMString::transcode(""), "", DOMString::transcode(""), aid, Effect::FIREWALLACTION) { 
    m_network = net;
    m_result  = status;
    m_fstatus_mode = true;
  }
  FirewallActionEffect( Network *net, int srchost, int srcport, 
                            int dsthost, int dstport, bool result, AttackID aid ):
                        Effect( "", DOMString::transcode(""), "", "", aid, Effect::FIREWALLACTION) { 
    
    m_network = net;
    m_rule.srchost = srchost;
    m_rule.dsthost = dsthost;
    m_rule.srcport = srcport;
    m_rule.dstport = dstport;
    m_result  = result;
    m_fstatus_mode = false;
  }
  ~FirewallActionEffect() {}
  
  virtual void Print(ostream& out) const;
  virtual void PrintSPIN(ostream& out) const {}
  
  long GetMemSize() const {
    return sizeof( FirewallActionEffect )  - sizeof(Effect);
  }
  FirewallRule GetFWRule() const {return m_rule; }
  bool GetResult() const { return m_result; }
  bool IsFWStatusEffect() const { return m_fstatus_mode; }
  
private:
  Network *m_network;
  FirewallRule m_rule;
  bool   m_result;
  bool   m_fstatus_mode;
};

class LTLAtomicProposition: public ModelMemUsage {
  
public:
  virtual ~LTLAtomicProposition() {}
  virtual void PrintSPIN(ostream& out) const = 0;
};

class LTLPrivilegeProposition: public LTLAtomicProposition{
  
public:
  LTLPrivilegeProposition( HostID host, const string &rel, const DOMString& value ) {
    m_host = host;
    m_rel  = rel;
    m_value = value;
  }
    
  virtual ~LTLPrivilegeProposition() {}
  virtual void PrintSPIN(ostream& out) const;
    
  long GetMemSize() const {
    return m_rel.size() + GetDOMStringSize( m_value ) + sizeof(LTLPrivilegeProposition);
  }
  
private:
  HostID    m_host;
  string    m_rel;
  DOMString m_value;
};

class LTLServiceProposition: public LTLAtomicProposition {
  
public:
  LTLServiceProposition(HostID host, const DOMString& service) {
    m_host    = host;
    m_service = service;
  }
  virtual ~LTLServiceProposition() {}

  virtual void PrintSPIN(ostream& out) const;
    
  long GetMemSize() const {
    return GetDOMStringSize( m_service ) + sizeof(LTLServiceProposition);
  }

private:
  HostID    m_host;
  DOMString m_service;
};

class LTLVulnerabilityProposition : public LTLAtomicProposition {

public:

  LTLVulnerabilityProposition(HostID host, const DOMString& vuln) {
    m_host = host;
    m_vuln = vuln;
  }
  virtual ~LTLVulnerabilityProposition() {}

  virtual void PrintSPIN(ostream& out) const;
    
  long GetMemSize() const {
    return GetDOMStringSize( m_vuln ) + sizeof(LTLVulnerabilityProposition);
  }

private:
  HostID    m_host;
  DOMString m_vuln;
};

class LTLKnowledgeProposition: public LTLAtomicProposition {

public:

  LTLKnowledgeProposition(const DOMString& know) {
    m_know_name    = know;
    m_value        = DOMString::transcode( "TRUE" );
  }
  LTLKnowledgeProposition(const DOMString& know, const DOMString& value) {
    m_know_name    = know;
    m_value        = value;
  }
  
  virtual ~LTLKnowledgeProposition() {}

  virtual void PrintSPIN(ostream& out) const;
    
  long GetMemSize() const {
    return GetDOMStringSize( m_know_name) + GetDOMStringSize( m_value ) +
           sizeof(LTLKnowledgeProposition);
  }

private:
  DOMString m_know_name;
  DOMString m_value;
};

class LTLObjectProposition: public LTLAtomicProposition {

public:

  LTLObjectProposition(HostID host_name, const DOMString& obj_name) { 
    m_host = host_name; m_obj_name = obj_name; 
  }
  virtual ~LTLObjectProposition() {}

  virtual void PrintSPIN(ostream& out) const;
  
  void   SetObjAttribute( const DOMString &attr_name, const DOMString &attr_value ) {
      m_obj_attributes[attr_name] = attr_value;
  }
  
  long GetMemSize() const {
    return GetDOMStringSize( m_obj_name) + GetDOMStringMapSize( m_obj_attributes ) + 
           sizeof(LTLObjectProposition);
  }
    
private:
  DOMString m_obj_name;
  HostID m_host;
  DOMStringMap m_obj_attributes;
};

class LTLSentObjectProposition: public LTLAtomicProposition {

public:

  LTLSentObjectProposition(HostID dsthost, HostID srchost, const DOMString& obj_name) { 
    m_dsthost = dsthost;
    m_srchost = srchost;
    m_obj_name = obj_name; 
    m_set_source = true;
    m_set_service = false;
  }
  
  LTLSentObjectProposition(HostID dsthost, const DOMString& service, const DOMString& obj_name) { 
    m_service = service; 
    m_dsthost = dsthost;
    m_obj_name = obj_name; 
    m_set_source = false;
    m_set_service = true;
  }
  
  LTLSentObjectProposition(HostID dsthost, HostID srchost, const DOMString& service, const DOMString& obj_name) { 
    m_service = service; 
    m_dsthost = dsthost;
    m_srchost = srchost;
    m_obj_name = obj_name; 
    m_set_source = true;
    m_set_service = true;
  }
  virtual ~LTLSentObjectProposition() {}

  virtual void PrintSPIN(ostream& out) const;
    
  long GetMemSize() const {
    return GetDOMStringSize( m_obj_name) + GetDOMStringSize( m_service ) + 
           sizeof(LTLSentObjectProposition);
  }
  
private:
  DOMString m_obj_name;
  HostID m_srchost;
  HostID m_dsthost;
  DOMString m_service; 
  bool m_set_source;
  bool m_set_service;
  //DOMStringMap m_obj_attributes;
};

class LTLConnectivityProposition: public LTLAtomicProposition {

public:

  LTLConnectivityProposition(HostID src_host, HostID dst_host) {
    m_use_service = false;
    m_service     = DOMString::transcode("");
    m_dst         = dst_host;
    m_src         = src_host;
  }
  LTLConnectivityProposition(HostID src_host, HostID dst_host, const DOMString& service) {
    m_use_service = true;
    m_service     = service;
    m_dst         = dst_host;
    m_src         = src_host;
  }
  virtual ~LTLConnectivityProposition() {}

  virtual void PrintSPIN(ostream& out) const;
    
  long GetMemSize() const {
    return GetDOMStringSize( m_service ) + sizeof(LTLConnectivityProposition);
  }

private:
  bool      m_use_service;
  DOMString m_service;
  HostID    m_dst;
  HostID    m_src;
};

class LTLTrustProposition: public LTLAtomicProposition {

public:

  LTLTrustProposition( HostID src_host, HostID dst_host) {
    m_dst = dst_host;
    m_src = src_host;
  }
  virtual ~LTLTrustProposition() {}

  virtual void PrintSPIN(ostream& out) const;
    
  long GetMemSize() const {
    return sizeof(LTLTrustProposition);
  }

private:
  HostID m_dst;
  HostID m_src;
};

typedef map<DOMString, LTLAtomicProposition*, less_str> LTLPropositionMap;

/* PDS: scanners.hpp */

struct VulnerabilityScanInfo
{
  string vuln_name;
  string vuln_description;
  set<string> CVEs;
  
  void set_CVEs_string( const string &cves )
  {
    string cve = "";
    unsigned int ind = 0, ind_start = 0;
    
    while ( ind < cves.length() )
    {
      for ( ;ind < cves.length() && (cves[ind] == ' ' || cves[ind] == '\t'); ind ++ );
      
      if ( ind < cves.length() )
      {
        ind_start = ind;
        ind = cves.find( ',', ind_start );
        cve = cves.substr( ind_start, ((ind == string::npos)? string::npos: ind - ind_start) );
        
        CVEs.insert( cve );
        
        if ( ind != string::npos )
          ind++;
      }
    }
  }
};

struct PortScanInfo
{
  unsigned int port_id;
  string   service_name;
  string   protocol_name;
  
  vector<VulnerabilityScanInfo*> detected_vulns;
  ~PortScanInfo()
  {
    for ( unsigned int i = 0; i < detected_vulns.size(); i++ )
      delete (VulnerabilityScanInfo*)detected_vulns[i];
  }
};

typedef set<PortScanInfo*> PortSet;

struct HostScanInfo
{
  string host_name;
  string host_ip;
  vector<PortScanInfo*> opened_ports;
  
  ~HostScanInfo()
  {
    for ( unsigned int i = 0; i < opened_ports.size(); i++ )
      delete (PortScanInfo*)opened_ports[i];
    opened_ports.clear();
  }
  
  StringSet* get_CVEs()
  {
    StringSet* cve_set = new StringSet();
    PortScanInfo  *port_info;
    VulnerabilityScanInfo*  vuln_info;
    
    for (unsigned int indx_port = 0; indx_port < opened_ports.size(); indx_port++)
    {
      port_info = opened_ports[indx_port];
      for (unsigned int indx_vuln = 0; indx_vuln < port_info->detected_vulns.size(); indx_vuln++)      
      {
        vuln_info = port_info->detected_vulns[indx_vuln];    
        cve_set->insert(vuln_info->CVEs.begin(), vuln_info->CVEs.end());
      }
    }
    
    return cve_set;
  }
  
  PortSet *get_services()
  {
    PortSet* svc_set = new PortSet();
    PortScanInfo  *port_info;
    
    for (unsigned int indx_port = 0; indx_port < opened_ports.size(); indx_port++)
    {
      port_info = opened_ports[indx_port];
      svc_set->insert(port_info);
    }
    
    return svc_set;
  }
  
  // for debuging
  void trace()
  {
    cout << host_name << " " << host_ip << endl;
    for (unsigned int i = 0; i < opened_ports.size(); i++)
    {
      cout << "\tPORT: " << opened_ports[i]->port_id << "; Service: " << opened_ports[i]->service_name << "; Protocol: " << opened_ports[i]->protocol_name << endl;
      for ( unsigned int j = 0; j < opened_ports[i]->detected_vulns.size(); j++ )
      {
        cout << "\t\tVuln. Name: " << opened_ports[i]->detected_vulns[j]->vuln_name << endl << "\t\tVuln. Descr: " << opened_ports[i]->detected_vulns[j]->vuln_description << endl;
        cout << "\t\tCVEs: ";
        if ( !opened_ports[i]->detected_vulns[j]->CVEs.empty() )
        {
          for ( set<string>::iterator p = opened_ports[i]->detected_vulns[j]->CVEs.begin(); p != opened_ports[i]->detected_vulns[j]->CVEs.end(); p++ )
              cout << *p << " ";
        }
        cout << endl;
        cout << endl;
        
      }
    }
  }
};


class UnknownHostError {
private:
  string errorString;

public:
  UnknownHostError(string error) : errorString(error) {}
  UnknownHostError() { UnknownHostError("Unknown host"); }
};

struct ScannedHost
{
  StringSet IP;
  PortSet services;
  StringSet CVEs;
};  

typedef map<string, ScannedHost*> ScannedHostMap;
typedef map<string, ScannedHost*>::iterator ScannedHostMapIter;

typedef list<string> StringList;
typedef list<string>::iterator StringListIter;

class Scanner
{
protected:
  ScannedHostMap  hosts;  
  string          scanner_name;  
  virtual bool  add_host_info(const string &host_name) = 0;

public:
  Scanner() {};
  virtual ~Scanner() {};

  virtual bool  set_register_data(const StringList &reg_strs) = 0;

  string        get_scanner_name() { return scanner_name; }
  StringSet*    getIP(const string &host_name) throw(UnknownHostError);
  PortSet*      getServices(const string &host_name) throw(UnknownHostError);
  StringSet*    getCVEs(const string &host_name) throw(UnknownHostError);
};

typedef map<string, Scanner*> ScannerMap;
typedef map<string, Scanner*>::iterator ScannerMapIter;

class ScannerManager
{
  ScannerMap  scanners;
public:
  ScannerManager() {};
  ~ScannerManager() 
  {
    for ( ScannerMapIter it = scanners.begin(); it != scanners.end(); it++ )
      if ( it->second != 0 )
        delete it->second;
    scanners.clear();
  };  
  
  void ScannerRegistration(Scanner *s)
  {
    scanners[s->get_scanner_name()] = s;
  }
  
  Scanner* get_scanner(const string &scanner_name)
  {
    ScannerMapIter  iter;
    
/*    cout << "Enter to the get_scanner with " << scanner_name << "\n";
    cout << "Scanners list:\n";
    for (iter = scanners.begin(); iter != scanners.end(); iter++)
      cout << "\tScanner name: " << (*iter).first << "\n";*/
    
    iter = scanners.find(scanner_name);
    
/*    bool yes = ((iter == scanners.end()) ? false : true);
    cout << "Is scanner found?  " << yes << "\n";*/
    
    if (iter != scanners.end())
      return (*iter).second;
    else
      return 0;
  }
};

/* PDS: nessus_scanner.hpp */

class NessusScanner : public Scanner
{
private:
  string  m_user_name;
  string  m_password;
  string  m_port;
  string  m_server_name;

protected:
  virtual bool  add_host_info(const string &host_name);    

public:
  NessusScanner();
  virtual ~NessusScanner() {};

  virtual bool  set_register_data(const StringList &reg_strs);
};

// Element types
enum ElementTypes {
  NETWORK,
  SERVICES,
  SERVICE,
  FTP,
  SSHD,
  DATABASE,
  VULNERABILITIES,
  VULNERABILITY,
  HOST,
  CONNECTIVITY,
  CVE,
  CVE1,  
  CVEID,
  REMOTE,
  TRUST,
  NO_TRUST,
  ADVERSARY,
  PRIVILEGES,
  PRIVILEGE,
  ATTACK,
  LOCAL_PRECONDITIONS,
  GLOBAL_PRECONDITIONS,
  TRACE,
  LOCAL_EFFECTS,
  GLOBAl_EFFECTS,
  IDS,
  PATH,
  PORT,
  HOSTS,
  ATTACKS,
  ID,
  IP,
  EMPTY,
  LEVEL,
  HOST1,
  HOST2,
  DETECTABLE,
  MODE,
  YES,
  NO,
  XML_TRUE,
  XML_FALSE,
  BOTH,
  NAME,
  LOCAL,
  TARGET,
  REL,
  GTE,
  EQ,
  LTE,
  LS,
  GT,
  VALUE,
  NONE,
  USER,
  ROOT,
  SOURCE,
  FROM,
  SERVICE_PORT_AGREEMENT,
  KNOWLEDGE,
  DESCRIPTION,
  TEXT,
  OBJECTS,
  OBJECT,
  OBJECT_ATTRIBUTE,
  SENDOBJECT,
  SENTOBJECT,
  SRCHOST,
  DSTHOST,
  DSTSVC,
  PASSARG,
  LTL_FORMULA,
  FORMULA,
  LTL_ATOMIC_PROPOSITION,
  
  //Firewalls
  FIREWALL,
  STATUS,
  RESULT,
  RULE,
  SRCPORT,
  DSTPORT,
  FIREWALL_REACHABLE,
  ON,
  OFF,
  ALLOW,
  DENY,
  TARGET_FW,
  WILDCARD_ALL,
  
  // Data sources
  OUTPOST_DS,
  ODBC_DS,
  END_MARKER
};

struct AuxilaryParserData
{
  // ElementTypes names in DOMString format
  DOMString enames[END_MARKER];
  
  // Data sources
#ifdef OUTPOST
  DBApi Outpost;
#endif
  
  ScannerManager scan_mang; //Scanner manager 
  NetInfo theNet; // state info for the network
  
  void Initialize();	/* enames[] */
};

class Network;

class Host: public ModelMemUsage
{
public:
  enum DFS_FLAGS {
    NONE_FLAG = 0,
    IN_STACK_FLAG = 1,
    PASSED_FLAG = 2
  };

  // Constructors and Destructor
  Host( Network *net ) { 
    network = net; 
    name.transcode(""); 
    ip_addrs = NULL; 
    netname = ""; 
    defined = false; 
    dfs_flags = NONE_FLAG;
  }
  Host( Network *net_obj, DOMString hostname, HostID hostid, const char *net = "") {
    network = net_obj;
    name = hostname;
    id = hostid;
    ip_addrs = NULL;
    netname = net;
    defined = false;
    dfs_flags = NONE_FLAG;
  }  
  ~Host() { Clear(); }

  // Accessors
  DOMString GetName() { return name; }
  HostID GetId() { return id; }

  bool ServiceRunning(const DOMString& svc) {
    return (services.find(svc) != services.end());
  }
  bool VulExists(const DOMString& vul) {
    return (vulnerabilities.find(vul) != vulnerabilities.end());
  }
  bool Connected(const DOMString& host, const DOMString& svc) {
    if (connectivity.count(host) == 0) return false;
    return (connectivity[host]->find(svc) != connectivity[host]->end());
  }
  bool Trusts(const DOMString& host) {
    return (trust.find(host) != trust.end());
  }
  StringSet *IPs() { return ip_addrs; }
  DOMStringSet *GetVulnerabilities() { return &vulnerabilities; }
  const char *NetworkName() { return netname; }
  bool IsDefined() { return defined; }
  void SetDefined(bool def) { defined = def; }
  
  unsigned char GetFlags() { return dfs_flags; }
  void SetFlags( unsigned char flags, bool enable ) {
    if ( enable )
      dfs_flags |= flags;
    else
      dfs_flags &= ~(flags);
  }

  // Generators (from XML)
  void GenerateIPs(DOM_Attr &attr);
  void GenerateServices(DOM_Element *element);
  void GenerateConnectivity(DOM_Element *element);
  void GenerateTrust(DOM_Element *element);
  void GenerateVulnerabilities(DOM_Element *element);
  void GenerateCVE(DOM_Element *element);
  void GenerateObjects(DOM_Element *element);

  void CountingServices(DOM_Element *element);
  void CountingConnectivity(DOM_Element *element);
  void CountingVulnerabilities(DOM_Element *element);
  void CountingCVE(DOM_Element *element);
    
  void SetDocument(DOM_Document *doc) { curr_doc = doc; }

  void SetIP(const string &ip) { 
    if (ip_addrs == 0)
      ip_addrs = new StringSet();
    
    ip_addrs->insert(ip);
  }
  
  HostObjectMap& GetHostObjects( ) { return objects; }
  
  long GetMemSize() const;
  
private:
  // Private functionality
  void GenerateConnectivityServices(DOM_Element *element, DOMStringSet *svcs);
  void CountingConnectivityServices(DOM_Element *element);
  void GenerateObject(DOM_Element *element);

  void Clear() {
    delete ip_addrs;
    map <DOMString, DOMStringSet *, less_str>::iterator conn_it;
    for ( conn_it = connectivity.begin(); conn_it != connectivity.end(); conn_it++ )
      if ( conn_it->second != 0 )
        delete (conn_it->second);
      
    connectivity.clear();
  }
    
private:
  Network *network;
  DOMString name;
  HostID id;
  StringSet *ip_addrs;
  const char *netname;
  DOMStringSet services;
  map <DOMString, DOMStringSet *, less_str> connectivity;
  DOMStringSet trust;
  DOMStringSet vulnerabilities;
  HostObjectMap objects;
  bool defined;
  unsigned char dfs_flags;

  // Document related to the network 
  DOM_Document  *curr_doc;
};


class Attack: public ModelMemUsage {

  enum AttackType {
    A_GLOBAL = 1,
    A_LOCAL  = 2,
    A_FIREWALL = 3
  };
public:
  Attack( Network *net ) {
    network = net;
    name.transcode("");
    detectable = No;
    type = A_GLOBAL;
    defined = false;
  }
  Attack( Network *net, DOMString& attackname, AttackID attackid ) {
    network = net;
    name = attackname;
    id = attackid;
    detectable = No;
    type = A_GLOBAL;
    defined = false;
  }
  
  ~Attack() {
    ClearPreconditionSet(locals);
    ClearPreconditionSet(globals);
    ClearPreconditionSet(derived);
  }

  enum Detectable {
    Yes,
    No,
    Varies
  };

  // Accessors
  bool IsLocal() { return (type == A_LOCAL); }
  bool IsGlobal() { return (type == A_GLOBAL); }
  bool IsAttackOnFirewall() { return (type == A_FIREWALL); }
  Detectable IsDetectable() { return detectable; }
  void SetLocal() { type = A_LOCAL; }
  void SetGlobal() { type = A_GLOBAL; }
  void SetAttackOnFirewall() { type = A_FIREWALL; }
  DOMString GetName() { return name; }
  void SetDescription(DOMString& desc) { description = desc; }
  DOMString GetDescription() { return description; }
  void AddCVE(DOMString &cveid) { cveids.insert(cveid); }
  
  DOMStringSet* GetCveids() { return &cveids; }
  
  // Generators (from XML)
  void GenerateLocalPreconditions(DOM_Element *element);
  void GenerateGlobalPreconditions(DOM_Element *element);
  void GenerateTrace(DOM_Element *element);
  void GenerateLocalEffects(DOM_Element *element);
  void GenerateGlobalEffects(DOM_Element *element);
  void GenerateDetectable(DOM_Element *element);

  void CountingPreconditions(DOM_Element *element);
  void CountingEffects(DOM_Element *element);
  
  bool IsDefined() { return defined; }
  void SetDefined(bool def) { defined = def; }
  
  long GetMemSize() const;

private:
  void GeneratePreconditions(DOM_Element *element, set <Precondition *>& preconditions);
  void GenerateEffects(DOM_Element *element);
  long GetPreconditionsMemSize( const set <Precondition *> &precond) const;

  void ClearPreconditionSet( set <Precondition *> &precond ) {
    set <Precondition *>::iterator it = precond.begin();
    for ( ; it != precond.end(); it++ )
      if ( *it != 0 )
        delete *it;
      
    precond.clear();
  }
  
private:
  Network   *network;
  DOMString name;
  HostID id;
  DOMString description;

  Detectable detectable;
  unsigned char type;
  bool defined;

  set <Precondition *> locals;
  set <Precondition *> globals;
  set <Precondition *> derived;

  vector <Effect *> effects;

  DOMStringSet cveids;

  friend class Network;
};

class LTLFormula: public ModelMemUsage
{
public:
  LTLFormula( ){ m_exist = false; }
    
  void Clear() {
    LTLPropositionMap::iterator pit = m_atomic_precond.begin();
    for ( ; pit != m_atomic_precond.end(); pit++ )
      delete pit->second;
    m_atomic_precond.clear();
  }
  
  ~LTLFormula(){ Clear(); }
  
  void GenerateLTL( Network *network, DOM_Element *element );
  bool GenerateSPIN( ostream& out );
  
  long GetMemSize() const;
  
private:
  void GenerateAtomicProposition( Network *network, DOM_Element *element );

private:
  LTLPropositionMap m_atomic_precond;
  DOMString         m_formula;
  bool              m_exist;
};

struct FirewallRule_less: public binary_function <FirewallRule, FirewallRule, bool> {
  bool operator () ( const FirewallRule &rule1, const FirewallRule &rule2 ) {
    return rule1.less( rule2 );
  }
};
  
typedef map< FirewallRule, bool, FirewallRule_less > FirewallRuleActionMap;
    
class Firewall: public ModelMemUsage {
  
public:
  Firewall( Network *network, const DOMString &name, FirewallID id, bool init_status ) {
    m_network = network;
    m_status = init_status;
    m_name = name;
    m_id = id;
  }
  
  ~Firewall(){ m_rules.clear(); }
  void SetLocation ( HostID host1, HostID host2 ) {
    m_host1 = host1;
    m_host2 = host2;
  }
  
  bool GetStatus() { return m_status; }
  DOMString GetName() { return m_name; }
  FirewallID GetId() { return m_id; }
    
  HostID GetFirstLocationHost() { return m_host1; }
  HostID GetSecondLocationHost() { return m_host2; }
  
  bool IsBetweenHosts( HostID host1, HostID host2 ) {
    return (m_host1 == host1 && m_host2 == host2 ||
            m_host1 == host2 && m_host2 == host1);
  }
  
  bool IsFirewallRuleDefined( const FirewallRule &rule ) {
    return (m_rules.find( rule ) != m_rules.end());
  }
  
  bool GetRuleAction( const FirewallRule &rule ) {
    if ( !IsFirewallRuleDefined(rule) )
      return false;
    return m_rules[ rule ];
  }
  
  bool AddRuleAction( const FirewallRule &rule, bool action = true ) {
    bool res = !IsFirewallRuleDefined( rule );
    
    if ( res ) {
      int add_flag = 0;
      FirewallRuleActionMap::iterator it = m_rules.begin();
      FirewallRuleActionMap::iterator tmp_it;
        
      for ( ; it != m_rules.end() && add_flag <= 0; ) {
        add_flag = it->first.MatchWildacrdRule( rule );
          
        if (add_flag == -1) {
          // rule includes it->first
          tmp_it = it++;
          m_rules.erase( tmp_it );
        }
        else  
          it++;
      }
        
      if ( add_flag <= 0 )
        m_rules[rule] = action;
    }
    
    return res;
  }
  
  FirewallRuleActionMap *GetRules() { return &m_rules; }
  
  long GetMemSize() const;
  
  void GenerateRules( DOM_Element *element );
  
private:
  Network   *m_network;
  
  DOMString  m_name;
  FirewallID m_id;
  //location
  HostID     m_host1;
  HostID     m_host2;
  
  bool      m_status;
  
  FirewallRuleActionMap m_rules;
};


typedef map<DOMString, Firewall*, less_str> NameFirewallMap;

class Adversary: public ModelMemUsage {

  friend class Network;
  friend class Attack;
  friend class LTLFormula;

public:
  Adversary() { network = 0; }
  ~Adversary() {}

  // Generators (from XML)
  void Generate(DOM_Element *element);

  DOMString GetPrivilege(DOMString& host) { return privileges[host]; }
  bool GetKnowledge(DOMString& ktype) { return knowledge[ktype]; }
  int GetPort(DOMString& svc) { return ports[svc]; }
  
  DOMStringSet GetKnowledgeNames() 
  {
    DOMStringSet res;
    for ( map<DOMString, bool, less_str>::const_iterator it = knowledge.begin(); it != knowledge.end(); it++ )
      res.insert( it->first );
    return res;    
  }
  
  long GetMemSize() const;
  
  void SetNetwork( Network *net ) { network = net;}
  
private:

  void GeneratePrivileges( DOM_Element *element);
  void GenerateServices(DOM_Element *element);
  void GenerateKnowledge(DOM_Element *element);

private:
  Network *network;
  // Service ports
  map<DOMString, int, less_str> ports;

  // Privilege levels
  map<DOMString, DOMString, less_str> privileges;

  // Knowledge
  map<DOMString, bool, less_str> knowledge;
};

class IntrusionDetectionSystem: public ModelMemUsage {

public:
  IntrusionDetectionSystem() { network = 0; }
  ~IntrusionDetectionSystem() {
    map <DOMString, DOMStringSet *, less_str>::iterator it;
    for ( it = paths.begin(); it != paths.end(); it++ )
      if ( it->second != 0 )
        delete (it->second);
      
    paths.clear();
  }

  bool Monitors(DOMString& host1, DOMString& host2) {
    bool monitors;
    map <DOMString, DOMStringSet *, less_str>::iterator src = paths.find(host1);
    monitors =
      ((src != paths.end()) ?
       (src->second->find(host2) != src->second->end()) :
       false);
    map <DOMString, DOMStringSet *, less_str>::iterator tgt = paths.find(host2);
    monitors = monitors ||
      ((tgt != paths.end()) ?
       (tgt->second->find(host1) != tgt->second->end()) :
       false);
    return monitors;
  }

  // Generators (from XML)
  void Generate(DOM_Element *element);
  
  long GetMemSize() const;
  
  void SetNetwork( Network *net ) { network = net;}

private:
  Network *network;
  // Paths monitored by the IDS
  map <DOMString, DOMStringSet *, less_str> paths;
};

typedef vector< vector<bool> > BoolVectorContainer;

class Network: public ModelMemUsage
{
public:

  // Constructors and Destructor
  Network();  
  ~Network() { Clear(); }

  // Accessors
  int GetNumHosts() { return nHosts; }
  int GetNumServices() { return nServices; }
  int GetNumAttacks() { return nAttacks; }

  void SetNumHosts(int numHosts) {
    if (numHosts < 0) return;
    nHosts = numHosts;
    hosts.resize(numHosts);
    for (HostID id = 0; id < numHosts; id++)
      hosts[id] = NULL;
    
    hid.clear();
  }
  void SetNumServices(int numServices) {
    if (numServices < 0) return;
    nServices = numServices;
    DOMString path = DOMString::transcode("path");
    sid2name.resize(nServices+1);
    sid2name[0] = path;
    
    sid.clear();
  }
  void SetNumAttacks(int numAttacks) {
    if (numAttacks < 0) return;
    nAttacks = numAttacks;
    attacks.resize(numAttacks);
    for (AttackID id = 0; id < (AttackID)numAttacks; id++)
      attacks[id] = NULL;
    
    aid.clear();
  }

  bool AddHost(DOMString& hostname, const string &ip, const char *net = "") {
    // Returns false if host by that name already exists, otherwise
    // adds the host
    if (hid.find(hostname) != hid.end()) return false;

    HostID newid = nHosts;
    hid[hostname] = newid;
    hosts.push_back(new Host(this, hostname, newid, net));
    hosts[nHosts]->SetIP(ip);
    nHosts++;
    return true;
  }

  bool AddAttack(DOMString& attackname) {
    // Returns false if attack by that name already exists, otherwise
    // adds the attack
    if (aid.find(attackname) != aid.end()) return false;

    AttackID newid = nAttacks;
    aid[attackname] = newid;
    attacks.push_back( new Attack( this, attackname, newid) );
    nAttacks++;
    return true;
  }

  bool AddService(DOMString& servicename, PortID port) {
    // Returns false if service by that name already exists, otherwise
    // adds the service
    if (sid.find(servicename) != sid.end()) return false;

    nServices++;
    ServiceID newid = nServices;
    sid[servicename] = newid;
    sid2name.resize(nServices+1);
    sid2name[newid] = servicename;
    ports[servicename] = port;

    return true;
  }
  
  bool DoesModelHaveFirewalls() { return !firewall_names.empty(); }
  
  bool AddFirewallName( const DOMString &name ) {
    if (firewall_names.find( name ) != firewall_names.end() )
      return false;
    
    firewall_names[name] = fid2name.size();
    fid2name.push_back( name );
    return true;
  }
  
  FirewallID GetFirewallNumber() { return fid2name.size(); }
  
  FirewallID GetFirewallID( const DOMString &name ) {
    if (firewall_names.find( name ) == firewall_names.end() )
      return 0;
    return firewall_names[name];
  }
  
  Firewall* GetFirewall( const DOMString &name ) {
    Firewall *f = 0;
    NameFirewallMap::iterator f_it;
    if ( (f_it = firewalls.find( name )) != firewalls.end() )
      f = f_it->second;
    
    return f;
  }
  
  Firewall* GetFirewall( FirewallID id ) {
    Firewall* f = 0;
    
    if ( id < fid2name.size() )
      f = GetFirewall( fid2name[id] );
    
    return f;
  }
  
  DOMString GetFirewallName( FirewallID id ) {
    if ( id >= fid2name.size() )
      return "";
    
    return fid2name[id];
  }
  
  bool IsFirewallDefined( const DOMString &name ) {
    return firewall_names.find( name ) != firewall_names.end();
  }
  
  bool AddFirewallPort( const DOMString &name ) {
    if (firewall_ports.find( name ) != firewall_ports.end() )
      return false;
    
    firewall_ports[name] = fport2name.size();
    fport2name.push_back( name );
    return true;
  }
  
  PortID GetFirewallPortID( const DOMString &name ) {
    if ( firewall_ports.find( name ) == firewall_ports.end() )
      return 0;
    return firewall_ports[name];
  }
  
  PortID GetFirewallPortNumber() { return fport2name.size(); }
  
  DOMString GetFirewallPortName( PortID id ) {
    if ( id >= fport2name.size() )
      return "";
    return fport2name[id];
  }
  
  bool IsFirewallPortDefined( const DOMString &name ) {
    return firewall_ports.find( name ) != firewall_ports.end();
  }
  
  int GetMaxFWHostPaths() { return max_path_count; }
  
  BoolVectorContainer* GetFWPathsBetweenHosts( HostID host1, HostID host2 ) {
    return &(pathes_firewalls[host1][host2]);
  }
  
  bool AreFWHostsConnected( HostID host1, HostID host2 ) {
    return (hostlinking[host1].find( host2 ) != hostlinking[host1].end());
  }    
  

  bool AddVul(DOMString& vulname) {
    // Returns false if vul by that name already exists, otherwise
    // adds the vul
    if (vulnerabilities.find(vulname) != vulnerabilities.end()) return false;

    vulnerabilities.insert(vulname);
    return true;
  }

  bool IsInitialized() {
    return (nHosts > 0) && (nServices > 0) && (nAttacks > 0);
  }

  DOMString& GetServiceName(ServiceID id) {
    assert(id <= (ServiceID)nServices);
    return sid2name[id];
  }
  
  bool IsHostAdded ( const DOMString& hostname ) {
    return hid.find(hostname) != hid.end();
  }
  
  HostID GetHostID(DOMString& hostname) {
    assert(hid.find(hostname) != hid.end());
    return hid[hostname];
  }
  
  AttackID GetAttackID(DOMString& attackname) {
    assert(aid.find(attackname) != aid.end());
    return aid[attackname];
  }
  
  void SetDocument(DOM_Document *doc) { curr_doc = doc; }
  // Generators (from XML)
  void GenerateHost(DOM_Element *element);
  void GenerateHosts(DOM_Element *element);
  void GenerateAttack(DOM_Element *element);
  void GenerateAttacks(DOM_Element *element);
  void GenerateServices(DOM_Element *element);
  void GenerateVulnerabilities(DOM_Element *element);
  void GenerateLTLFormula( DOM_Element *element ) { ltl_formula.GenerateLTL( this, element ); }
  void GenerateFirewall( DOM_Element *element );

  // Generators to SMV
  void GenerateSMV(ostream& out, string& target);
  
  long GetMemSize() const;

private:
  void GenerateAttackDeclarationsSMV(ostream& out);
  void GenerateHostInitializationsSMV(ostream& out);
  void GenerateRelationInitializationsSMV(ostream& out);
  void GenerateAttackIdsSMV(ostream& out);
  void GenerateTransitionsSMV(ostream& out);
  void GenerateDefinesSMV(ostream& out);
  void GenerateInvarSMV(ostream& out);
  void GenerateHostSMV(ostream& out);
  void GenerateHostRelationsSMV(ostream& out);
  void GenerateAdversarySMV(ostream& out);
  void GenerateIDSSMV(ostream& out);
  void GenerateAttacks(ostream& out);
  void GenerateTransitionSMV(ostream& out, vector <Effect *>& effects,
                             const char *varname, HostID tgt, const char *entityname = NULL, 
                             const char *subentityname = NULL);
  
  char* GenerateWebAttack(DOM_Element *element);
  bool AttackURLParse(const string& url, string& host_name, int& port, string& path);
  void GenerateObjAttrValueDefinesSMV( ostream& out );
  
public:
  // Generators to SPIN
  void GenerateSPIN(ostream& out, string& target);

    
private:
  void CountingPass(DOM_Element *element);
  void CountingHost(DOM_Element *element);
  void CountingAttack(DOM_Element *element);
  void CountingFirewall(DOM_Element *element);
  void CountingFirewallRulePorts( DOM_Element *element );

  void GenerateDefinesSPIN(ostream& out);
  void GenerateAttackPreconditionsSPIN(ostream& out);
  void GenerateAttackEffectsSPIN(ostream& out);
  void GenerateTypeDefsSPIN(ostream& out);
  void GenerateMainProcessSPIN(ostream& out);
  void GenerateNeverClaimSPIN(ostream& out, string& target);
  void GenerateEffectsSPIN(ostream& out, vector <Effect *>& effects,
			     const char *varname, HostID tgt, const char *entityname = NULL);

  void GenerateFWReachabilityComputationSPIN( ostream& out );
  void GenerateFWRuleSPIN( ostream& out, const FirewallRule &rule, bool action, int firewall = -1 );
  void GenerateFWExplicitRuleSPIN( ostream& out, HostID srchost, PortID srcport, HostID dsthost, PortID dstport, 
                                           bool action, int firewall = -1 );

  
  // Output a PAA-style packet specification that host hid1 can reach host hid2
  void OutputIPPacketsSpec(ostream& out, const char *predicate, HostID hid1, HostID hid2);
  // Test if host hid1 can reach host hid2 on port
  bool Reachable(const char *netspec, HostID hid1, HostID hid2);
  
  // Output a PAA-style packet specification that host hid1 can reach host hid2
  void OutputIPPacketsSpec(ostream& out, const char *predicate, HostID hid1, HostID hid2, PortID port);
  // Test if host hid1 can reach host hid2 on port
  bool Reachable(const char *netspec, HostID hid1, HostID hid2, PortID port);
  
  //Generate named constants for object attribute values
  DOMString GetAttrConstant( const DOMString &value );
  void GenerateObjAttrValueDefinesSPIN( ostream& out );
  long GetEffectsMemSize( const vector<Effect*> &eff ) const;
  
  void ClearEffectVector( vector< Effect* > &eff ) {
    for ( unsigned int i = 0; i < eff.size(); i++ ) {
      delete eff[i];
      eff[i] = 0;
    }
    eff.clear();
  }
  
  void Clear() {
    unsigned int i;
    for ( i = 0; i < hosts.size(); i++ ) {
      delete (Host*)hosts[i];
      hosts[i] = 0;
    }
    hosts.clear();
    for ( i = 0; i < attacks.size(); i++ ) {
      delete (Attack*)attacks[i];
      attacks[i] = 0;
    }    
    attacks.clear();
    
    NameFirewallMap::iterator f_it = firewalls.begin();
    for ( ; f_it != firewalls.end(); f_it++ )
      delete (Firewall*)(f_it->second);
    firewalls.clear();
    
    ClearEffectVector( priveff );
    ClearEffectVector( svceff );
    ClearEffectVector( vuleff );
    ClearEffectVector( conneff );
    ClearEffectVector( trusteff );
    ClearEffectVector( knoweff );
    ClearEffectVector( objeff );
    ClearEffectVector( passargeff );
    ClearEffectVector( firewalleff );
  }
  
  void ComputeFConnectivity();
  
private:
  AuxilaryParserData *auxilary;

public:
  // Global variables
  Adversary adversary; // Hacker description
  IntrusionDetectionSystem ids; // IDS description

private:
  int nHosts;
  int nServices;
  int nAttacks;

  // Document related to the network 
  DOM_Document  *curr_doc;
  // Map from host names to host ids
  map<DOMString, HostID, less_str> hid;  
  // Map from host ids to hosts
  vector<Host *> hosts;

  // Map from attack names to attack ids
  map<DOMString, AttackID, less_str> aid;  
  // Map from attack ids to attacks
  vector<Attack *> attacks;

  // Service ports
  map<DOMString, PortID, less_str> ports;
  // Service ids
  map<DOMString, ServiceID, less_str> sid;
  vector<DOMString> sid2name;

  // Vulnerabilities
  DOMStringSet vulnerabilities;
  
  //Names of host objects/attributes
  HostObjectAttrNameMap host_objects;
  //All object attribute values
  DOMStringSet          obj_attr_values;
  
  //LTL formula
  LTLFormula            ltl_formula;
  
  /////////////////////////////////
  unsigned int          max_path_count;
  vector< vector<BoolVectorContainer> > pathes_firewalls;
  
  vector<HostIdSet>     hostlinking;
  vector<DOMString>     fport2name;
  NamePortIdMap         firewall_ports;
  
  vector<DOMString>     fid2name;
  map< DOMString, FirewallID, less_str>  firewall_names;
  NameFirewallMap       firewalls;
  /////////////////////////////////

  // Attack effects on various state parts
  vector <Effect *> priveff;
  vector <Effect *> svceff;
  vector <Effect *> vuleff;
  vector <Effect *> conneff;
  vector <Effect *> trusteff;
  vector <Effect *> knoweff;
  vector <Effect *> objeff;
  vector <Effect *> passargeff;
  vector <Effect *> firewalleff;

  friend class Attack;
  friend class Host;
  friend class Firewall;
  friend void GenerateNetwork( AuxilaryParserData *auxilary, Network *network, DOM_Element *element );
  friend class LTLFormula;
  friend class Adversary;
  friend class IntrusionDetectionSystem;
    
public:
  Host*   GetHost(HostID id)
  {
    if (id < (HostID)nHosts)
      return hosts[id];
    else
      return 0;
  }
  
  Attack* GetAttack(AttackID id)
  {
    if (id < (AttackID)nAttacks)
      return attacks[id];
    else
      return 0;
  }
  
  DOMStringSet* GetVulnerabilities() { return &vulnerabilities; }
  HostObjectAttrNameMap* GetHostObjectNames() { return &host_objects; }
  DOMStringSet* GetHostObjectValues() { return &obj_attr_values; }
  NameFirewallMap* GetFirewalls() { return &firewalls; }
  
};

extern char *AttackNameDefineStr(const DOMString &aname);
extern char *FixUpCVEname(char *cvename);

int atoi(DOMString dstr);
ostream& operator<< (ostream& target, const Precondition *pre);
ostream& operator<< (ostream& target, const Effect *eff);

/* PDS: nessus_scaninfo.hpp */

HostScanInfo* ParseNessusXML( const string &file_name );

/* PDS: hostnameip.h */

string HostNameToIP( const string &name );
string HostIPToName( const string &ip );

/* PDS: ErrorReporter.cpp */

class DOMTreeErrorReporter : public ErrorHandler
{

public:
    // -----------------------------------------------------------------------
    //  Constructors and Destructor
    // -----------------------------------------------------------------------
    DOMTreeErrorReporter()
    {
    }

    ~DOMTreeErrorReporter()
    {
    } 


    // -----------------------------------------------------------------------
    //  Implementation of the error handler interface
    // -----------------------------------------------------------------------
    void warning(const SAXParseException& toCatch);
    void error(const SAXParseException& toCatch);
    void fatalError(const SAXParseException& toCatch);
    void resetErrors();
};
