#include <assert.h>
#include <cstdio>

#include <iostream>
#include <fstream>
#include <sstream>

#include <string>
#include <list>
#include <vector>
#include <map>
#include <ext/hash_set>

using namespace std;

typedef unsigned char uchar;
typedef int QueueRef;

class StateBase;

/*
	PDS: the rest of the world sees "HASH_KEY_BYTE_SIZE",
	"max_st_arr_size", "pack_state_hash_key", and "hash_func".
	This stuff should not be in a class.

	PDS:  Don't we just want:
	
		type hash
		val hash : State -> hash
		val eq : hash * hash -> bool
	
	What gets hashed?   The output of autom.cpp:/get_state_byte_array.
	
	There is another copy of jenkin's hash lying in the code somewhere.
*/

class StateHash
{
	static uchar PearsonX[256];
	static uchar DWORD_BYTE_PER_KEY;
	
	static void jenkins_hash(uchar *st_arr, int arr_size, uchar *key);
	static void pearson_hash(uchar *st_arr, int arr_size, uchar *key);

public:

	/* PDS: Could take an argument indicating the type of hash. */
	static void init_hashing();
	
	enum HashFunc { JENKINS_FUNC = 0, PEARSON_FUNC };
	static HashFunc hash_function;

	/* Number of bytes in a hash value. */
	static uchar HASH_KEY_BYTE_SIZE;
	
	/*
		"returns the size of key".  no larger
		than HASH_KEY_BYTE_SIZE and if smaller,
		padded with zero bytes.
		
		PDS: Should be char* hash(void*, size_t);
		(No need to mention State.)
	*/
	static uchar pack_state_hash_key(StateBase* st, uchar **key);

	/* Largest state ever hashed. */
	static int max_st_arr_size;
};

class BoolArr{

	friend uchar pack_state_hash_key(StateBase *st, uchar **key);
	
private:
	uchar *m_arr;
	unsigned int m_bit_count;

public:
	BoolArr();
	BoolArr(BoolArr *arr);
	BoolArr(int size);	// initially all false

	~BoolArr();

	void set_size(int size);	// resize and make all false
	int get_size();	// PDS: m_bit_count should be public

	void set_var(int ind, bool val);
	bool is_var_enabled(int ind);

	bool operator!=(BoolArr &other);
	BoolArr& operator=(const BoolArr &other);
	
	void debug_print();
	void copy_bit_arr(uchar *copy_to);
};
  
void make_state_attr_format(unsigned long format_flags, ostream *output);	/* autom.cpp */

class CAutomBase;
class QueueEl;
class Queue;
  
typedef map<string, char> TypeStorage;
typedef map<string, long> NameStorage;
typedef TypeStorage::const_iterator TypeCI;
typedef NameStorage::const_iterator NameCI;

class Tracer
{
public:
	static const unsigned long ASCII_OUTPUT;
	static const unsigned long PRINT_TO_STR;
	static const unsigned long LEFTY_INPUT;
	static const unsigned long GRAPH_ATTR_FORMAT;
	static const unsigned long BIN32;
	
	static const unsigned long TRACE_STATE;
	static const unsigned long TRACE_QUEUE;
	static const unsigned long TRACE_QUEUE_EL;
	static const unsigned long TRACE_AUTOM;
	static const unsigned long ATTR_INIT;

	static const unsigned long EXPAND_QUEUE;
	static const unsigned long TRACE_MEM;
	static const unsigned long TRACE_TABLE;
	static const unsigned long TRACE_LOC_VAR;
	static const unsigned long TRACE_GL_VAR;
	static const unsigned long TRACE_VAR;
	static const unsigned long FULL_TRACE;

	static const unsigned long ACTION_DEL;
	static const unsigned long ACTION_ADD;
	static const unsigned long ACTION_CLONE;
	static const unsigned long ACTION_VAR;
	static const unsigned long ACTION_LOOK;
	static const unsigned long ACTION_FLAGS;
	
	static string ATTR_START;
	static string ATTR_END;

private:
	static ostringstream m_out_sstream;
	static ostream *m_out_stream;

protected:
	static unsigned long m_tracing_flags;

	static const uchar QUEUE_CLASS;
	static const uchar QUEUE_EL_CLASS;
	static const uchar AUTOM_CLASS;
	static const uchar STATE_CLASS;

	static const int MAX_ASCII_LINE_LENGTH;

	static string EOL;
	static string TAB;
	
	static NameStorage m_name_set;
	static vector<string> m_name_arr;
	static TypeStorage m_type_set;
	static vector<string> m_type_arr;
	static int m_curr_line_len;

public:
	static void write_1bytes(char, ostream *);
	static void write_2bytes(short, ostream *);
	static void write_4bytes(long, ostream *);
	static void write_bin_identifier(long id, ostream *);

protected:
	virtual uchar get_tracer_class_id();

	static char add_type(const string &type);
	static long add_name(const string &name);
   
public:   
	static char get_type_id(const string& type);
	static long get_name_id(const string& name);
   
	static long calc_names_storage_size();
	static void clear_names_storage();
	
	static long name_number();
	static void write_names_storage(ostream*);

protected:
	static void print(const string &prefix, long,  const string &suffix="");
	static void print(const string &prefix, const string &, const string &suffix="");
	static void print(long, const string &suffix="");
	static void print(const string &);

public:   
	static void finish_ASCII_block(bool is_end = true);

protected:
	static void print_attr(const string&  type, const string &name="", long value=-1);
	static void print_attr_ind(const string&  type, long name, long value=-1);
   
public:
	virtual void trace(unsigned long trace_action, long extra_info=0, const string &line_prefix="");

	static void put_error(const string &s);

	static unsigned long get_flags();
	static void set_flags(unsigned long tracing_flags);
   
	static ostream *get_curr_stream();
   
	static void init_tracing(ostream *out_stream, unsigned long flags=0xFFFFFFFF);

	static string get_output_str();

	static void trace(unsigned long trace_action, Tracer* tracer, long l_info=0, const string &line_prefix="", const string &extra_info="");

	static void trace(Tracer* tracer, long l_info=0, const string &line_prefix="", const string &extra_info="");
};

class Clonable
{
public:
	virtual Clonable* clone() = 0;
};

class QueueTypeCarrier :
	public Clonable
{
public:
	virtual QueueEl* new_element() = 0;
	virtual int get_type() = 0;
};

/*
	Channel elements.
*/
class QueueEl :
	public Clonable,
	public Tracer
{
public:
	virtual ~QueueEl();

protected:
	uchar get_tracer_class_id();
	virtual void  trace_fld(unsigned long trace_action, long extra_info=0, const string &line_prefix="");

public:
	void  trace(unsigned long trace_action, long extra_info=0, const string &line_prefix="");
	virtual void set(int count, ...)=0;
	virtual bool get(int ind, ...)=0;
	virtual bool equal(string fld_mask, QueueEl *el)=0;
	virtual bool equal_fld(int ind, ... /*var*/)=0;
	virtual int fld_number()=0;
	virtual int get_type()=0;
};

/*
	Channels.
	
	PDS: It looks like these are stacks, not queues.
*/
class Queue :
	public Clonable,
	public Tracer 
{

private:
	int	m_owner;       /* id of process */
	unsigned int	m_max_length;	/* max length of queue */
	QueueTypeCarrier*	m_type_carrier;
	vector<QueueEl*>	m_items;	/* each item structure depends on type of queue */
	bool	m_is_enable;	/* is it currently in use */
	bool	m_rvz;

public:
	Queue(unsigned int max_el, QueueTypeCarrier* q_type_el);
	~Queue();
	Clonable* clone();
	QueueEl* create_el();

protected:
	uchar get_tracer_class_id();

public:
	void trace(unsigned long trace_action, long extra_info=0, const string &line_prefix="");

	bool is_empty();
	bool is_full();
	int get_len();
	void set_enable(bool);
	bool get_enable();
	bool is_rvz();
	int get_type();

	/*
		Stack access.
	*/
	bool send(QueueEl* el);	/* push el, false on type mismatch (w/o adding el) */
	bool receive(QueueEl **el);	/* pop top of stack to el, false if empty */
	/*
		Stack indices for everything but del(): Index |this|
		(0) addresses the stack's top (bottom).
	*/
	bool look(int ind, QueueEl **el);	/* false if index out of bounds */
	bool look(QueueEl **el);	/* top of stack, false if empty */
	/*
		Scan stack from top to bottom, finding the index of
		the first element x satisfying x->equal(mask,el).  -1
		if not found.
	*/
	int has(string mask, QueueEl *el);
	/*
		Remove an element (and free it).  Index 0 (|this|)
		addresses the stack's top (bottom).
	*/
	void del(int ind);

	long calc_size();
};

class StateBase :
	public Clonable,
	public Tracer 
{
public:
	/* for debugging */
	static int state_in_memory;
	static int max_state_in_memory;
	static unsigned int max_state_size;
	
protected:
	vector<CAutomBase*>	m_autom_list;	/* all automata including each proctype */
	CAutomBase	*m_ltl;	/* never claim (negated LTL formula) */	
	uchar	m_flags;
	
public:
	StateBase();
	virtual ~StateBase();
	
protected:
	void delete_automs();
	void copy_to(StateBase *st);

public:
	virtual Clonable* clone()=0;

protected:
	uchar get_tracer_class_id();

public:
	virtual void trace(unsigned long trace_action, long extra_info=0, const string &line_prefix="");
	unsigned int calc_size();
	virtual int get_state_byte_array(uchar **arr)=0;
	
	bool is_stutter();	/* current transition is a stuter move (spin's tau&128) */
	void set_stutter(bool);
	
	bool is_progress();	/* this is a progress state (spin's o_pm&4) */
	void set_progress(bool);
	
	bool is_timeout();	/* timeout enabled (spin's tau&1) */
	void set_timeout(bool);
	
	bool is_case_found();	/* correct statement chosen in if..fi (spin's o_pm&1) */
	void set_is_case_found(bool);

	/* PDS: get_boq and set_boq are stubs only */
	virtual int  get_boq();
	virtual void set_boq(int);

	CAutomBase* get_never_claim();

	int add_proc(CAutomBase *autom);	/* returns autom's pid */
	CAutomBase* get_proc(int pid);	/* 0 if pid invalid */
	int proc_count();	/* NB first discards processes s.t. get_curr_state() is NULL */

	bool get_enabled_flag(int pid);	/* True if any of pid's transitions is enabled. */
	bool get_progress_flag();
	int  get_curr_state(int pid);
};

struct Trans{
	short	atom;	/* type of the trans: atomic or local */
	int	st;	/* the next state */
	int	this_st;	/* transition's owner */
	string	tp;	/* src txt of statement */
	/*
		Transition id, unique within proc.  (Index in
		m_stt_table array (begins at 1) or 0.)
	*/
	int	t_id;
	int	forw;	/* index forward transition */
	Trans	*nxt;	/* next transition, for non-deterministic state */

	Trans();
	Trans(Trans*);
};

class CAutomBase :
	public Clonable,
	public Tracer
{
	friend class StateBase;
public:
	static const uchar ATOM;
	static const uchar ND_ATOM;
protected:
	static const uchar L_ATOM;
	static const uchar D_ATOM;

	enum{
		NOT_STOP,
		FIRST_STOP,
		CONTINUE_STOP
	};
   
	StateBase	*m_owner;
	short	_pid;
	short	m_curr_state;
	int	m_state_number;
	uchar	m_flags;
	uchar	m_stop;
	
protected:
	CAutomBase();
	virtual bool set_trans_table()=0;
	bool set_state_number(int);	/* sets state count */
	
	virtual ~CAutomBase();

	uchar get_tracer_class_id();
	void trace_autom_base(unsigned long trace_action, long extra_info, const string &name, const string &line_prefix);
	static void trace_autom_end(unsigned long trace_action, const string &name, const string &line_prefix);
	static void trace_table(unsigned long trace_action, Trans *** table, int sst_len, bool *accpstate, bool *progstate, long extra_info, const string &line_prefix);

	bool is_test_enabled();
   
public:
	bool is_moved();
	void set_moved(bool);

	bool is_first_skip_trans();
	void set_first_skip_trans(bool);
   
	bool is_inside_atomic();
	void set_inside_atomic(bool);
   
	bool is_else_passed();
	void set_else_passed(bool);

	void set_pid(int pid);

	int get_curr_state();
	int get_state_count();
	bool set_curr_state(int new_state);
	bool is_enable();

	bool is_stop_found_first();
   
	virtual bool is_state_accepted(int st)=0;
	virtual bool is_state_final(int st)=0;
	
	virtual bool is_state_in_progress(int st)=0;
	virtual int  get_autom_byte_array(uchar *arr)=0;

	/*
		Nonzero id of first transition for state st.  Zero if error.
	*/
	virtual int get_first_trans_id(int st)=0;
	/*
		Nonzero id of next transition after trans_id from its
		state.  Zero if no more.
	*/
	virtual int get_next_trans_id(int trans_id)=0;

	bool is_trans_enable(StateBase*, int trans_id);

	/*
		Change the current (PDS: or given?) state to a new one
		using the specified transition.
	*/
	virtual bool new_state(StateBase *state, int trans_id)=0;

private:
	void del_transition(Trans *t);
	
	void imed(bool *accpstate, bool *progstate, int st1, int st2);	/* set intermediate state */

protected:
	void rebuild_table(Trans ***sst_table, bool *accpstate, bool *progstate, int sst_len, int &init_st);

	void del_table(Trans ***sst_table, int &table_ref, int &table_length);

	/*
		Transition table initialization:
		
		start_table_init(_,_,#states); add_transition+; end_table_init
	*/
	bool start_table_init(Trans ***table, int ref_count, int table_len);
   
	bool add_transition(Trans ***sst_table, int ref_count, int table_len,
		int t_id,	/* state id (index in the transition array) */
		short atom,	/* atom falg */
		int st,	/* next state */
		int forw,	/* forward transition */
		int backw,	/* backward transition */
		string tp,	/* src txt of statement (comment) */
		int glob,	/* global flag */
		int tpe0,	/* [0] class of operation (for reduction) */
		int tpe1);	/* [1] class of operation (for reduction) */

	/* For initializing deadlock states. */
	bool add_transition(Trans ***table, int ref_count, int table_len,
		int t_id,	/* state id (index in the transition array) */
		string tp);	/* src text of statement (comment) */

	bool add_transition(Trans ***table, int ref_count, int table_len,
		int t_id,	/* state id (index in the transition array) */
		short atom,	/* atom flag */
		int st,	/* next state */
		int forw);	/* forward transition */

	void end_table_init(Trans ***table, bool *accptstate, bool *progstate, int &ref_count, int &table_len, int &init_st);

};

StateBase* initial_state();
