/***** spin: pangen1.c *****/

/* Copyright (c) 1991-2000 by Lucent Technologies - Bell Laboratories     */
/* All Rights Reserved.  This software is for educational purposes only.  */
/* Permission is given to distribute this code provided that this intro-  */
/* ductory message is not removed and no monies are exchanged.            */
/* No guarantee is expressed or implied by the distribution of this code. */
/* Software written by Gerard J. Holzmann as part of the book:            */
/* `Design and Validation of Computer Protocols,' ISBN 0-13-539925-4,     */
/* Prentice Hall, Englewood Cliffs, NJ, 07632.                            */
/* Send bug-reports and/or questions to: gerard@research.bell-labs.com    */

#include "spin.h"
#ifdef PC
#include "y_tab.h"
#else
#include "y.tab.h"
#endif
#include "pangen1.h"
#include "pangen3.h"
#include <stdlib.h>

extern FILE	*tc, *th, *tt;
extern Label	*labtab;
extern Ordered	*all_names;
extern ProcList	*rdy;
extern Queue	*qtab;
extern Symbol	*Fname;
extern int	lineno, verbose, Pid, separate;
extern int	nrRdy, nqs, mst, Mpars, claimnr, eventmapnr;
extern short	has_sorted, has_random, has_provided;

int	Npars=0, u_sync=0, u_async=0;

static int var_ind = 0;

static Symbol	*LstSet=ZS;
static int	acceptors=0, progressors=0, nBits=0;
static int	Types[] = { UNSIGNED, BIT, BYTE, CHAN, MTYPE, SHORT, INT, STRUCT };

static void	dohidden(void);
static void	do_init(FILE *, Symbol *, int);
static void	tc_predef_np(void);

static void
reverse_names(ProcList *p)
{
	if (!p) return;
	reverse_names(p->nxt);
	fprintf(th, "   \"%s\",\n", p->n->name);
}

void
gensvmap(void)
{
	ntimes(tc, 0, 1, SvMap);
}

static struct {
	char *s,	*t;		int n,	m,	p;
} ln[] = {
	{"accept",	"m_accpstate",	6,	1,	0},
	{0,		0,		0,	0,	0},
};

void
end_labs(FILE *tc, Symbol *s, int i)
{	int oln = lineno;
	Symbol *ofn = Fname;
	Label *l;
	int j; char foo[128];

	if ((i == claimnr && separate == 1)
	||  (i != claimnr && separate == 2))
		return;

	for (l = labtab; l; l = l->nxt)
	for (j = 0; ln[j].n; j++)
		if (strncmp(l->s->name, ln[j].s, ln[j].n) == 0
		&&  strcmp(l->c->name, s->name) == 0)
		{	fprintf(tc, "\t\t%s[%d] = true;\n",
				ln[j].t, l->e->seqno);
			acceptors += ln[j].m;
			progressors += ln[j].p;
			if (l->e->status & D_ATOM)
			{	sprintf(foo, "%s label inside d_step", ln[j].s );
				lineno = l->e->n->ln;
				Fname  = l->e->n->fn;
				printf("generator: %3d:%s, warning, %s - is invisible\n",
					lineno, Fname?Fname->name:"-", foo);
			}
			if (j > 0 && (l->e->status & ATOM))
			{	sprintf(foo, "%s label inside atomic", ln[j].s);
					lineno = l->e->n->ln;
				Fname  = l->e->n->fn;
				printf("generator: %3d:%s, warning, %s - is invisible\n",
					lineno, Fname?Fname->name:"-", foo);
			}
		}	
	lineno = oln;
	Fname  = ofn;
}

void
calc_end_labs(char *accpstate, char *progstate, char *finalstate, Symbol *s, int i)
{	int oln = lineno;
	Symbol *ofn = Fname;
	Label *l;
	int j; char foo[128];

	if ((i == claimnr && separate == 1)
	||  (i != claimnr && separate == 2))
		return;

	for (l = labtab; l; l = l->nxt)
	for (j = 0; ln[j].n; j++)
		if (strncmp(l->s->name, ln[j].s, ln[j].n) == 0
		&&  strcmp(l->c->name, s->name) == 0)
		{	if (ln[j].m)
			{	accpstate[l->e->seqno] = 1;
				if (strcmp(l->s->name, "accept_all") == 0 )
					finalstate[l->e->seqno] = 1;
			} else if (ln[j].p)
				progstate[l->e->seqno] = 1;
		}
	lineno = oln;
	Fname  = ofn;

}

void
ntimes(FILE *fd, int n, int m, char *c[])
{
	int i, j;
	for (j = 0; c[j]; j++)
	for (i = n; i < m; i++)
	{	fprintf(fd, c[j], i, i, i, i, i, i);
		fprintf(fd, "\n");
	}
}

void
prehint(Symbol *s)
{	Lextok *n;

	printf("spin: warning, ");
	if (!s) return;

	n = (s->context != ZS)?s->context->ini:s->ini;
	if (n)
	printf("line %3d %s, ", n->ln, n->fn->name);
}

void
checktype(Symbol *sp, char *s)
{	char buf[128]; int i;

	if (!s
	|| (sp->type != BYTE
	&&  sp->type != SHORT
	&&  sp->type != INT))
		return;

	if (sp->hidden&16)	/* formal parameter */
	{	ProcList *p; Lextok *f, *t;
		int posnr = 0;
		for (p = rdy; p; p = p->nxt)
			if (p->n->name
			&&  strcmp(s, p->n->name) == 0)
				break;
		if (p)
		for (f = p->p; f; f = f->rgt) /* list of types */
		for (t = f->lft; t; t = t->rgt, posnr++)
			if (t->sym
			&&  strcmp(t->sym->name, sp->name) == 0)
			{	checkrun(sp, posnr);
				return;
			}

	} else if (!(sp->hidden&4))
	{	if (!(verbose&32)) return;
		sputtype(buf, sp->type);
		i = strlen(buf);
		while (buf[--i] == ' ') buf[i] = '\0';
		prehint(sp);
		if (sp->context)
			printf("proctype %s:", s);
		else
			printf("global");
		printf(" '%s %s' could be declared 'bit %s'\n",
			buf, sp->name, sp->name);
	} else if (sp->type != BYTE && !(sp->hidden&8))
	{	if (!(verbose&32)) return;
		sputtype(buf, sp->type);
		i = strlen(buf);
		while (buf[--i] == ' ') buf[i] = '\0';
		prehint(sp);
		if (sp->context)
			printf("proctype %s:", s);
		else
			printf("global");
		printf(" '%s %s' could be declared 'byte %s'\n",
			buf, sp->name, sp->name);
	}
}

static int bool_count = 0;
static int chan_count = 0;

int
dolocal(FILE *ofd, char *pre, int dowhat, char *s)
{	int h, j, k=0; extern int nr_errs;
	Ordered *walk;
	Symbol *sp;

	bool_count = 0;
	for (j = 0; j < 8; j++)
	for (h = 0; h <= 1; h++)
	for (walk = all_names; walk; walk = walk->next)
	{	sp = walk->entry;
		if (sp->context
		&& !sp->owner
		&&  sp->type == Types[j]
		&&  ((h == 0 && sp->nel == 1) || (h == 1 && sp->nel > 1))
		&&  strcmp(s, sp->context->name) == 0)
		{	switch (dowhat) {
			case INIV:
				checktype(sp, s); /* fall through */
				if (sp->hidden&16) { k++; break; }
			case PUTV:
				do_var(ofd, 0, dowhat, pre, sp, "\t", " = ", ";\n", 1);
				k++;
				break;
			case TRACEV:
				do_var(ofd, 0, dowhat, "", sp, "\t", pre, "\";\" + EOL", 1);
				k++;
				break;
			case SETBOOL:
				k++;
				do_var(ofd, 0, dowhat, "", sp, "", pre, "", 1);
				break;
			case GETBOOL:
				k++;
				do_var(ofd, 0, dowhat, "", sp, "", pre, "", 1);
				break;
			case PUTBOOLV:
				k++;
				do_var(ofd, 0, dowhat, pre, sp, "\t", " = ", ";\n", 1);
				break;
			case BOOLV:
				k++;
				do_var(0, 0, dowhat, "", sp, "", pre, "", 1);
				break;
			case BOOLCOUNT:
				do_var(0, 0, dowhat, "", sp, "", pre, "", 1);
				break;
			case CHANCOUNT:
				do_var(0, 0, dowhat, "", sp, "", pre, "", 1);
				break;
			}
			if (strcmp(s, ":never:") == 0)
			{	printf("error: %s defines local %s\n",
					s, sp->name);
				nr_errs++;
	}	}	}
	if (dowhat == BOOLCOUNT)
		return bool_count;
	if (dowhat == CHANCOUNT)
		return chan_count;
	return k;
}

int
doglobal(FILE *tc, FILE *tt, char *pre, int dowhat)
{	Ordered *walk;
	Symbol *sp;
	int j, cnt = 0;

	bool_count = 0;
	chan_count = 0;
	var_ind = 2;
	for (j = 0; j < 8; j++)
	for (walk = all_names; walk; walk = walk->next)
	{	sp = walk->entry;
		if (!sp->context
		&&  !sp->owner
		&&  sp->type == Types[j])
		{	if (Types[j] != MTYPE || !ismtype(sp->name))
			switch (dowhat) {
			case INIV:
				checktype(sp, (char *) 0);
				/* fall through */
			case PUTV:
				cnt++; 
				do_var(tc, 0, dowhat, pre, sp, "\t", " = ", ";\n", 0);
				break;
			case SETBOOL:
				cnt++;
				do_var(tc, 0, dowhat, "", sp, "", pre, "", 0);
				break;
			case GETBOOL:
				cnt++;
				do_var(tc, 0, dowhat, "", sp, "", pre, "", 0);
				break;
			case PUTBOOLV:
				cnt++; 
				do_var(tc, 0, dowhat, pre, sp, "\t", " = ", ";\n", 0);
				break;
			case TRACEV:
				cnt++; 
				do_var(tc, tt, dowhat, "", sp, "\t", pre, "\";\" + EOL", 0);
				break;
			case BOOLV:
				cnt++;
				do_var(0, 0, dowhat, "", sp, "", pre, "", 0);
				break;
			case BOOLCOUNT:
				do_var(0, 0, dowhat, "", sp, "", pre, "", 0);
				break;
			case CHANCOUNT:
				do_var(0, 0, dowhat, "", sp, "", pre, "", 0);
				break;
	}	}	}
	switch(dowhat) {
	case BOOLCOUNT:
		return bool_count;
	case CHANCOUNT:
		return chan_count;
	}
	return cnt;
}

int
haslocal(char *s)
{	int j, k=0; extern int nr_errs;
	Ordered *walk;
	Symbol *sp;

	for (walk = all_names; walk; walk = walk->next)
	{	sp = walk->entry;
		for ( j = 0; j < 8; j++ )
			if (sp->context
			&&  !sp->owner
			&&  sp->type == Types[j]
			&&  strcmp(s, sp->context->name) == 0)
				return 1;
	}
	return 0;
}

extern int bool_ind;

int
do_var(FILE *ofd, FILE *tfd, int dowhat, char *s, Symbol *sp,
	char *pre, char *sep, char *ter, int islocal)
{	int i, arr_range, count;
	char *type;

	arr_range = 0;
	type = get_var_type_string(sp, &arr_range);
	count = 1;
	switch(dowhat) {
	case PUTV:
		if (sp->type != BIT)
		{	if (arr_range > 0)
				fprintf(ofd, "%s%s %s[%d]%s",
					pre, type, sp->name, arr_range, ter);
			else
				fprintf(ofd, "%s%s %s%s",
					pre, type, sp->name, ter);
		}
		break;
	case PUTBOOLV:
		if (sp->type == BIT)
		{	if (arr_range > 0)
				fprintf(ofd, "%s%s %s[%d]%s",
					pre, "int", sp->name, arr_range, ter);
			else
				fprintf(ofd, "%s%s %s%s",
					pre, "int", sp->name, ter);
		} else if (sp->type == STRUCT)
		{	if (arr_range > 0)
				fprintf(ofd, "%s%s_bool %s[%d]%s",
					pre, type, sp->name, arr_range, ter);
			else
				fprintf(ofd, "%s%s_bool %s%s",
					pre, type, sp->name, ter);
		}
		break;

	case INIV:
		if (sp->type == STRUCT)
		{	/* struct may contain a chan */
			walk_struct(ofd, tfd, dowhat, s, sp, pre, sep, ter, islocal);
			break;
		}
		if (!sp->ini)	/* it defaults to 0 */
			break;
		if (sp->type == BIT)
			break;
		if (sp->nel == 1)
		{	if (sp->type == CHAN
			||  sp->ini->ntyp != CONST
			||	sp->ini->val != 0 )
			{	fprintf(ofd, "%s%s%s%s", pre, s, sp->name, sep);
				do_init(ofd, sp, islocal);
				fprintf(ofd, "%s", ter);
			}
		} else
			for (i = 0; i < sp->nel; i++)
			{	if (sp->type == CHAN
				||  sp->ini->ntyp != CONST
				||  sp->ini->val != 0)
				{	fprintf(ofd, "%s%s%s[%d]%s",
						pre, s, sp->name, i, sep);
					do_init(ofd, sp, islocal );
					fprintf(ofd, "%s", ter);
			}	}
		break;
	case SETBOOL:
		if (sp->type == STRUCT)
		{	/* struct may contain a chan */
			walk_struct(ofd, tfd, dowhat, s, sp, pre, sep, ter, islocal);
		} else if (sp->type == BIT)
		{	if (sp->nel <= 1)
			{	fprintf( ofd, "\t%s%s%s = %i;\n",
					((islocal)? "": ""), s, sp->name, bool_ind++ );
			} else
			{	for (i = 0; i < sp->nel; i++)
					fprintf(ofd, "\t%s%s%s[%d] = %i;\n",
						((islocal)? "": ""), s, sp->name, i, bool_ind++);
		}	}
		break;
	case GETBOOL:
		if (sp->type == STRUCT)
		{	/* struct may contain a chan */
			walk_struct(ofd, tfd, dowhat, s, sp, pre, sep, ter, islocal);
		} else if (sp->type == BIT)
		{	if (sp->nel <= 1)
			{	fprintf(ofd, "\t\tarr.set_var(%i, %s.%s%s > 0);\n",
					bool_ind++, ((islocal)? "bool_locals": "bool_globals"), s, sp->name);
			} else
			{	for (i = 0; i < sp->nel; i++)
					fprintf(ofd, "\t\tarr.set_var(%i, %s.%s%s[%d] > 0);\n",
						bool_ind++, ((islocal)? "bool_locals": "bool_globals"), s, sp->name, i);
		}	}
		break;
	case TRACEV:
		if (sp->type == STRUCT)
		{	/* struct may contain a chan */
			walk_struct(ofd, tfd, dowhat, s, sp, pre, sep, ter, islocal);
		}
		else if (sp->type == CHAN)
		{	if (sp->nel <= 1)
			{	fprintf(ofd, "%sif (trace_queue && (trace_action & GRAPH_ATTR_FORMAT) == 0 )\n",
					pre);
				fprintf(ofd, "%s\tq = %sget_queue(%s.%s%s);\n",
					pre, ((islocal)? "state->": ""), ((islocal)? "locals": "globals"), s, sp->name);
				fprintf(ofd, "%sif ( q != 0 )\n%s{\n",
					pre, pre);
				fprintf(ofd, "%s\tprint( %s + \" chan: %s%s=\" );\n",
					pre, sep, s, sp->name);
				fprintf(ofd, "%s\tq->trace( trace_action | ACTION_VAR | TRACE_QUEUE, %s.%s%s, %s + TAB );\n",
					pre, ((islocal)? "locals": "globals"), s, sp->name, sep);
				fprintf(ofd, "%s}\n",
					pre);
				fprintf(ofd, "%selse\n",
					pre);
				if (tfd != 0)
				{	fprintf(tfd, "%s%s\n",
						s, sp->name);
					fprintf(ofd, "%s\tprint_attr_ind(chan_type, %d, %s.%s%s - 1);\n",
						pre, var_ind++, ((islocal)? "locals": "globals"), s, sp->name);
				} else
					fprintf(ofd, "%s\tprint_attr(\"chan\", \"%s%s\", %s.%s%s - 1);\n",
						pre, s, sp->name, ((islocal)? "locals": "globals"), s, sp->name);
			}
			else
			{	for (i = 0; i < sp->nel; i++)
				{	fprintf(ofd, "%sif ( trace_queue && (trace_action & GRAPH_ATTR_FORMAT) == 0 )\n",
						pre);
					fprintf(ofd, "%s\tq = %sget_queue(%s.%s%s[%d]);\n",
						pre, ((islocal)? "state->": ""), ((islocal)? "locals": "globals"), s, sp->name, i);
					fprintf(ofd, "%sif (q != 0)\n%s{\n",
						pre, pre);
					fprintf(ofd, "%s\tprint(%s + \" chan: %s%s[%d]=\" );\n",
						pre, sep, s, sp->name, i);
					fprintf(ofd, "%s\tq->trace(trace_action | ACTION_VAR | TRACE_QUEUE, %s.%s%s[%d], %s + TAB);\n",
						pre, ((islocal)? "locals": "globals"), s, sp->name, i, sep);
					fprintf(ofd, "%s}\n", pre);
					fprintf(ofd, "%selse\n", pre);
					if (tfd != 0)
					{	fprintf(tfd, "%s%s[%d]\n",
							s, sp->name, i);
						fprintf(ofd, "%s\tprint_attr_ind(chan_type, %d, %s.%s%s - 1);\n",
							pre, var_ind++, ((islocal)? "locals": "globals"), s, sp->name, i);
					} else
						fprintf(ofd, "%s\tprint_attr(\"chan\", \"%s%s[%d]\", %s.%s%s[%d] - 1);\n",
							pre, s, sp->name, i, ((islocal)? "locals": "globals"), s, sp->name, i);
		}	}	}
		else 
		{	char *str_type = get_base_type_str(sp->type);
			if (sp->nel <= 1)
			{	if (sp->type != BIT)
				{	if (tfd != 0)
					{	fprintf(tfd, "%s%s\n",
							s, sp->name);
						fprintf(ofd, "\tprint_attr_ind(%s_type, %d, %s.%s%s );\n",
							((strcmp(str_type, "uchar") == 0)? "byte": str_type),
							var_ind++, ((islocal)? "locals": "globals"), s, sp->name);
					} else
						fprintf(ofd, "\tprint_attr(\"%s\", \"%s%s\", %s.%s%s);\n",
							((strcmp(str_type, "uchar") == 0)? "byte": str_type),
							s, sp->name, ((islocal)? "locals": "globals"), s, sp->name);
				} else
				{	if (tfd != 0)
					{	fprintf(tfd, "%s%s\n",
							s, sp->name);
						fprintf(ofd, "\tprint_attr_ind(bool_type, %d, %sbit_arr.is_var_enabled(%s.%s%s));\n",
							var_ind++, ((islocal)? "autom_": ""),
							((islocal)? "locals_bool": "globals_bool"), s, sp->name);
					} else
						fprintf(ofd, "\tprint_attr(\"%s\", \"%s%s\", %sbit_arr.is_var_enabled(%s.%s%s) );\n",
							"bool", s, sp->name, ((islocal)? "autom_": ""),
							((islocal)? "locals_bool": "globals_bool"), s, sp->name);
			}	}
			else
			{	for (i = 0; i < sp->nel; i++)
				{	if (sp->type != BIT)
					{	if (tfd != 0)
						{	fprintf(tfd, "%s%s[%d]\n",
								s, sp->name, i);
							fprintf(ofd, "\tprint_attr_ind(%s_type, %d, %s.%s%s[%d]);\n",
								((strcmp( str_type, "uchar") == 0)? "byte": str_type),
								var_ind++, ((islocal)? "locals": "globals"), s, sp->name, i);
						} else
							fprintf(ofd, "\tprint_attr(\"%s\", \"%s%s[%d]\", %s.%s%s[%d]);\n",
								((strcmp(str_type, "uchar") == 0)? "byte": str_type),
								s, sp->name, i, ((islocal)? "locals": "globals"), s, sp->name, i);
					} else
					{	if (tfd != 0)
						{	fprintf(tfd, "%s%s[%d]\n",
								s, sp->name, i );
							fprintf(ofd, "\tprint_attr_ind(bool_type, %d, %sbit_arr.is_var_enabled(%s.%s%s[%d]));\n",
								var_ind++, ((islocal)? "autom_": ""),
								((islocal)? "locals_bool": "globals_bool"), s, sp->name, i);
						}
						else
							fprintf(ofd, "\tprint_attr( \"%s\", \"%s%s[%d]\", %sbit_arr.is_var_enabled(%s.%s%s[%d]));\n",
								"bool", s, sp->name, i, ((islocal)? "autom_": ""),
								((islocal)? "locals_bool": "globals_bool"), s, sp->name, i);
		}	}	}	}
		break;
	case BOOLCOUNT:
		if (sp->type == BIT)
		{	bool_count += count * sp->nel;
			count = 1;
			break;
		} else if (sp->type == STRUCT)
			walk_struct(ofd, tfd, dowhat, s, sp, pre, sep, ter, islocal);
		if (sp->nel > 1)
			count *= sp->nel;
		break;

	case CHANCOUNT:
		if (sp->type == CHAN)
		{	chan_count += count * sp->nel;
			count = 1;
			break;
		} else if (sp->type == STRUCT)
			walk_struct(ofd, tfd, dowhat, s, sp, pre, sep, ter, islocal);
		if (sp->nel > 1)
			count *= sp->nel;
		break;
	}
	free(type);
	return 0;
}

static void
do_init(FILE *ofd, Symbol *sp, int islocal)
{	int i; extern Queue *ltab[];
	char *q_class;

	if (sp->ini
	&&  sp->type == CHAN
	&&  ((i = qmake(sp)) > 0))
	{	q_class = get_q_type_carrier_class_name(ltab[i-1]->qcid);
		if (sp->ini->ntyp == CHAN)
			fprintf(ofd, "%sadd_queue(%d, new %s())",
				((islocal)? "state->": ""), ltab[i-1]->nslots, q_class);
		else
		{	/*for variables and constants??????????*/
			putstmnt(ofd, sp->ini, 0);
		}
		free(q_class);
	}
	else
		putstmnt(ofd, sp->ini, 0);
}

static int
blog(int n)	/* for small log2 without rounding problems */
{	int m=1, r=2;

	while (r < n) { m++; r *= 2; }
	return 1+m;
}

static void
tc_predef_np(void)
{	int i = nrRdy;	/* 1+ highest proctype nr */

	fprintf(th, "#define _NP_	%d\n", i);
	if (separate == 2) fprintf(th, "extern ");
	fprintf(th, "uchar reached%d[3];  /* np_ */\n", i);

	fprintf(th, "#define nstates%d	3 /* np_ */\n", i);
	fprintf(th, "#define endstate%d	2 /* np_ */\n\n", i);
	fprintf(th, "#define start%d	0 /* np_ */\n", i);

	fprintf(tc, "\tcase %d:	/* np_ */\n", i);
	if (separate == 1)
	{	fprintf(tc, "\t\tini_claim(%d, h);\n", i);
	} else
	{	fprintf(tc, "\t\t((P%d *)pptr(h))->_t = %d;\n", i, i);
		fprintf(tc, "\t\t((P%d *)pptr(h))->_p = 0;\n", i);
		fprintf(tc, "\t\treached%d[0] = 1;\n", i);
		fprintf(tc, "\t\taccpstate[%d][1] = 1;\n", i);
	}
	fprintf(tc, "\t\tbreak;\n");
}

Element *
huntstart(Element *f)
{	Element *e = f;

	if (e->n)
	{	if (e->n->ntyp == '.' && e->nxt)
			e = e->nxt;
#if 0
		else if (e->n->ntyp == ATOMIC
		     ||  e->n->ntyp == D_STEP
		     ||  e->n->ntyp == NON_ATOMIC)
			e->n->sl->this->last->nxt = e->nxt;
#endif
		else if (e->n->ntyp == UNLESS)
			return e->sub->this->frst;
	}
	return e;
}

Element *
huntele(Element *f, int o, int stopat)
{	Element *g, *e = f;
	int cnt; /* a precaution against loops */

	if (e)
	for (cnt=0; cnt < 10 && e->n; cnt++)
	{
		if (e->seqno == stopat)
			break;

		switch (e->n->ntyp) {
		case GOTO:
			g = get_lab(e->n,1);
			cross_dsteps(e->n, g->n);
			break;
		case '.':
		case BREAK:
			if (!e->nxt)
				return e;
			g = e->nxt;
			break;
		case UNLESS:
			g = huntele(e->sub->this->frst, o, stopat);
			break;
#if 0
		case D_STEP:
		case ATOMIC:
		case NON_ATOMIC:
			e->n->sl->this->last->nxt = e->nxt;
#endif
		default:	/* fall through */
			return e;
		}
		if ((o & ATOM) && !(g->status & ATOM))
			return e;
		e = g;
	}
	return e;
}

static void
ncases(FILE *fd, int p, int n, int m, char *c[])
{	int i, j;

	for (j = 0; c[j]; j++)
	for (i = n; i < m; i++)
	{	fprintf(fd, c[j], i, p, i);
		fprintf(fd, "\n");
	}
}

char*
get_var_type_string(Symbol *sp, int *arr_range)
{	char *ret_str;
	char type_name[256];

	ret_str = 0;
	*arr_range = 0;
	memset(type_name, 0, sizeof(type_name));
	switch (sp->type) {
	case UNSIGNED:
		if (sp->hidden&1)
			sprintf(type_name, "uchar");
		else
			sprintf(type_name, "uint");
		break;
	case BIT:
		if (sp->nel == 1 && !(sp->hidden&1))
		{	sprintf(type_name, "uchar");
			break;
		} /* else fall through */
	case MTYPE:
	case BYTE:
		sprintf(type_name, "uchar");
		break;
	case CHAN:
		sprintf(type_name, "QueueRef");
		break;
	case SHORT:
		sprintf(type_name, "short");
		break;
	case INT:
		sprintf(type_name, "int");
		break;
	case STRUCT:
		if (!sp->Snm)
			fatal("undeclared structure element %s", sp->name);
		sprintf(type_name, "struct %s", sp->Snm->name);
		break;
	case PREDEF:
		return ret_str;
	default:
		fatal("variable %s undeclared", sp->name);
	}
	ret_str = emalloc(strlen(type_name) + 1);
	memcpy(ret_str, type_name, strlen(type_name));
	if (sp->nel != 1)
		*arr_range = sp->nel;
	return ret_str;
}

/*
	Generate queue element class name.
*/
char *
get_queue_el_class_name(short cid)
{	char *q_prefix, *c_name;
	char name[255];
	
	q_prefix = "Queue";
	memset(name, 0, sizeof(name));
	sprintf(name, "%s%d", q_prefix, cid);
	c_name = emalloc(strlen(name) + 1);
	sprintf(c_name, "%s", name);
	return c_name;
}

char *
get_q_type_carrier_class_name(short cid)
{
	char *q_prefix, *q_suffix, *c_name;
	char name[255];
	
	q_prefix = "Queue";
	q_suffix = "TypeCarrier";
	memset(name, 0, sizeof(name));
	sprintf(name, "%s%d%s", q_prefix, cid, q_suffix);
	c_name = emalloc(strlen(name) + 1);
	if (c_name != 0)
		sprintf(c_name, "%s", name);
	return c_name;
}

char *
get_base_type_str(int type)
{
	switch (type) {
	case BIT:
	case MTYPE:
	case BYTE:
		return "uchar";
	case CHAN:
		return "QueueRef";
	case SHORT:
		return "short";
	case INT:
		return "int";
	}
	return "";
}

/*
	Generate queue element classes.
*/
void
gen_queue_el_classes(FILE *th, FILE *tc, Queue *qtab)
{	int j;
	Queue *q;
	char *q_class_name, *q_carrier_class_name;
	
	for (q = qtab; q; q = q->nxt)
	{	if (q->qid != q->qcid)
			continue;
		q_class_name = get_queue_el_class_name(q->qcid);
		q_carrier_class_name = get_q_type_carrier_class_name(q->qcid);
		fprintf(th, "class %s : public QueueEl\n", q_class_name);
		fprintf(th, "{\n");
		fprintf(th, "private:\n");
		fprintf(th, "\tstatic const short fld_count;\n");
		fprintf(th, "public:\n");
		fprintf(th, "\tstatic const short type_id;\n");
		for (j = 0; j < q->nflds; j++)
		{	switch (q->fld_width[j]) {
			case BIT:
				fprintf(th, "\tuchar fld%d;\t/* BIT */\n", j);
				break;
			case MTYPE:
				fprintf(th, "\tuchar fld%d;\t/* MTYPE */\\n", j);
				break;
			case BYTE:
				fprintf(th, "\tuchar fld%d;\t/* BYTE */\n", j);
				break;
			case CHAN:
				fprintf(th, "\tQueueRef fld%d;\t/* CHAN */\n", j);
				break;
			case SHORT:
				fprintf(th, "\tshort fld%d;\n", j);
				break;
			case INT:
				fprintf(th, "\tint fld%d;\n", j);
				break;
			default:
				fatal("bad channel spec ", "");
			}
		}
		fprintf(th, "\t%s()\n\t{\n", q_class_name);
		for (j = 0; j < q->nflds; j++)
			fprintf(th, "\t\tfld%d = 0;\n", j);
		fprintf(th, "\t}\n");
		fprintf(th, "\t~%s() { }\n", q_class_name);
		fprintf(th, "\tvoid set(int, ...);\n");
		fprintf(th, "\tbool get(int, ...);\n");
		fprintf(th, "\tbool equal(string, QueueEl *);\n");
		fprintf(th, "\tbool equal_fld(int, ...);\n");
		fprintf(th, "\tint fld_number() { return fld_count; }\n");
		fprintf(th, "\tint get_type() { return type_id; }\n");
		fprintf(th, "\tClonable *clone();\n");
		fprintf(th, "\n\tvoid trace_fld(unsigned long, long extra_info = 0, const string &line_prefix = \"\");\n");

		fprintf(th, "};\n");
		fprintf(th, "const short %s::fld_count = %d;\n", q_class_name, q->nflds);
		fprintf(th, "const short %s::type_id = %d;\n\n", q_class_name, q->qcid);

		fprintf(th, "class %s : public QueueTypeCarrier\n", q_carrier_class_name);
		fprintf(th, "{\n");
		fprintf(th, "public:\n");
		fprintf(th, "\t%s() {}\n", q_carrier_class_name);
		fprintf(th, "\t~%s() {}\n", q_carrier_class_name);
		fprintf(th, "\tint get_type() { return %s::type_id; }\n", q_class_name);
		fprintf(th, "\tQueueEl *new_element() { return new %s(); }\n", q_class_name);
		fprintf(th, "\tClonable *clone() { return new %s(); }\n", q_carrier_class_name);
		fprintf(th, "};\n\n");

		fprintf(tc, "void %s::set(int count, ...)\n", q_class_name);
		fprintf(tc, "{\n");
		fprintf(tc, "\tint fld_count = min(count, fld_number());\n");
		fprintf(tc, "\tva_list args;\n");
		fprintf(tc, "\tva_start(args, count);\n\n");
		fprintf(tc, "\tfor (int i = 0; i < fld_count; i++)\n");
		fprintf(tc, "\t{\n");
		fprintf(tc, "\t\tswitch (i)\n");
		fprintf(tc, "\t\t{\n");
		for (j = 0; j < q->nflds; j++)
		{	char *str_type = get_base_type_str(q->fld_width[j]);
			fprintf(tc, "\t\tcase %d:\n", j);
			if (strcmp(str_type, "uchar") == 0)
				fprintf(tc, "\t\t\tfld%d = (uchar)va_arg(args, int);\n", j);
			else
				fprintf(tc, "\t\t\tfld%d = va_arg(args, %s);\n", j, str_type);
			fprintf(tc, "\t\t\tbreak;\n");
		}
		fprintf(tc, "\t\t}\n");
		fprintf(tc, "\t}\n");
		fprintf(tc, "\n\tva_end(args);\n");
		fprintf(tc, "}\n\n");

		/* void get(int ind, ...)*/
		fprintf(tc, "bool %s::get(int ind, ...)\n", q_class_name);
		fprintf(tc, "{\n");
		fprintf(tc, "\tva_list args;\n\n");
		fprintf(tc, "\tif (ind >= fld_number() || ind < 0)\n");
		fprintf(tc, "\t\treturn false;\n\n");
		fprintf(tc, "\tva_start(args, ind);\n\n");
		fprintf(tc, "\tswitch(ind)\n");
		fprintf(tc, "\t{\n");
		for (j = 0; j < q->nflds; j++)
		{	char *str_type = get_base_type_str(q->fld_width[j]);
			fprintf(tc, "\tcase %d:\n", j);
			fprintf(tc, "\t\t{\n");
			fprintf(tc, "\t\t%s *tmp = va_arg(args, %s*);\n", str_type, str_type);
			fprintf(tc, "\t\t*tmp = fld%d;\n", j);
			fprintf(tc, "\t\t}\n");
			fprintf(tc, "\t\tbreak;\n");
		}
		fprintf(tc, "\t}\n");
		fprintf(tc, "\tva_end(args);\n\n");
		fprintf(tc, "\treturn true;\n");
		fprintf(tc, "}\n\n");

		fprintf(tc, "bool %s::equal(string fld_mask, QueueEl *el)\n", q_class_name);
		fprintf(tc, "{\n");
		fprintf(tc, "\tbool res = true;\n");
		fprintf(tc, "\tif (el == 0)\n");
		fprintf(tc, "\t\treturn false;\n\n");
		fprintf(tc, "\tif (get_type() != el->get_type())\n");
		fprintf(tc, "\t\treturn false;\n\n");
		fprintf(tc, "\tint fld_count = min(fld_number(), (int)(fld_mask.length()));\n\n");
		fprintf(tc, "\t%s *the_el = dynamic_cast<%s*>(el);\n\n", q_class_name, q_class_name);
		fprintf(tc, "\tif (the_el == 0)\n");
		fprintf(tc, "\t\treturn false;\n\n");
		fprintf(tc, "\tfor (int i = 0; (i < fld_count) && res; i++)\n");
		fprintf(tc, "\t{\n");
		fprintf(tc, "\t\tif (fld_mask[i] == '1')\n");
		fprintf(tc, "\t\t{\n");
		fprintf(tc, "\t\t\tswitch(i)\n");
		fprintf(tc, "\t\t\t{\n");
		for (j = 0; j < q->nflds; j++)
		{	fprintf(tc, "\t\t\tcase %d:\n", j);
			fprintf(tc, "\t\t\t\tif (fld%d != the_el->fld%d)\n", j, j);
			fprintf(tc, "\t\t\t\t\tres = false;\n");
			fprintf(tc, "\t\t\t\tbreak;\n");
		}
		fprintf(tc, "\t\t\t}\n");
		fprintf(tc, "\t\t}\n");
		fprintf(tc, "\t}\n\n");
		fprintf(tc, "\treturn res;\n");
		fprintf(tc, "}\n\n");

		fprintf(tc, "bool %s::equal_fld(int ind, ...)\n", q_class_name);
		fprintf(tc, "{\n");
		fprintf(tc, "\tbool res = false;\n");
		fprintf(tc, "\tva_list args;\n\n");
		fprintf(tc, "\tif (ind >= fld_number() || ind < 0)\n");
		fprintf(tc, "\t\treturn false;\n\n");
		fprintf(tc, "\tva_start(args, ind);\n\n");
		fprintf(tc, "\tswitch(ind)\n");
		fprintf(tc, "\t{\n");
		for (j = 0; j < q->nflds; j++)
		{	char *str_type = get_base_type_str(q->fld_width[j]);
			fprintf(tc, "\tcase %d:\n", j);
			if (strcmp(str_type, "uchar") == 0)
				fprintf(tc, "\t\tres = (fld%d == (uchar)va_arg(args, int));\n", j);
			else
				fprintf(tc, "\t\tres = (fld%d == va_arg(args, %s));\n", j, str_type);
			fprintf(tc, "\t\tbreak;\n");
		}
		fprintf(tc, "\t}\n");
		fprintf(tc, "\tva_end(args);\n\n");
		fprintf(tc, "\treturn res;\n");
		fprintf(tc, "}\n\n");

		fprintf(tc, "Clonable *%s::clone()\n", q_class_name);
		fprintf(tc, "{\n");
		fprintf(tc, "\t%s *the_clone = new %s();\n\n", q_class_name, q_class_name);
		fprintf(tc, "\tif (the_clone != 0)\n");
		fprintf(tc, "\t\tthe_clone->set(%d, ", q->nflds);
		for (j = 0; j < q->nflds; j++)
		{	if (j != 0)
				fprintf(tc, ", fld%d", j);
			else
				fprintf(tc, "fld%d", j);
		}
		fprintf(tc, ");\n\n");
		fprintf(tc, "\treturn the_clone;\n");
		fprintf(tc, "}\n\n");

		fprintf(tc, "void %s::trace_fld(unsigned long trace_action, long extra_info, const string &line_prefix)\n", q_class_name);
		fprintf(tc, "{\n");
		fprintf(tc, "\tprint(line_prefix + \"{\");\n");
		for (j = 0; j < q->nflds; j++)
		{	if (q->fld_width[j] == CHAN)
				fprintf(tc, "\tprint(\" chan: fld%d=\", fld%d, \"%s\");\n", j, j, "; ");
			else
			{	char *str_type = get_base_type_str(q->fld_width[j]);
				fprintf(tc, "\tprint(\" %s: fld%d=\", fld%d, \"%s\");\n", ((strcmp(str_type, "uchar") == 0)? "byte": str_type), j, j, ";");
		}	}
		fprintf(tc, "\tprint(\"}\" + EOL);\n");
		fprintf(tc, "}\n\n");

		free(q_carrier_class_name);
		free(q_class_name);
	}
}
