#include <libsgspin.h>

uchar StateHash::HASH_KEY_BYTE_SIZE = 8;
uchar StateHash::PearsonX[256];
int   StateHash::max_st_arr_size = 0;
StateHash::HashFunc StateHash::hash_function = StateHash::JENKINS_FUNC;
uchar StateHash::DWORD_BYTE_PER_KEY = 2;

void
StateHash::init_hashing()
{
	int i, j;
	uchar k, s;
	
	for(i=0; i<256; i++)
		PearsonX[i] = i;

	k = 7;
	for(j=0; j<3; j++)
		for (i=0; i<256; i++){
			s = PearsonX[i];
			k = (k + s) & 0xFF;
			PearsonX[i] = PearsonX[k];
			PearsonX[k] = s;
		}

	DWORD_BYTE_PER_KEY = (HASH_KEY_BYTE_SIZE / 4) * 4;
}

#define jmix(a,b,c) \
{ \
	a -= b; a -= c; a ^= (c>>13); \
	b -= c; b -= a; b ^= (a<<8); \
	c -= a; c -= b; c ^= (b>>13); \
	a -= b; a -= c; a ^= (c>>12);  \
	b -= c; b -= a; b ^= (a<<16); \
	c -= a; c -= b; c ^= (b>>5); \
	a -= b; a -= c; a ^= (c>>3);  \
	b -= c; b -= a; b ^= (a<<10); \
	c -= a; c -= b; c ^= (b>>15); \
}

void
StateHash::jenkins_hash(uchar *st_arr, int arr_size, uchar *key)
{
    register size_t a,b,c;
    int      i, j;
    size_t   rest;
        
    /* Set up the initial state */
    rest = a = b = 0x9e3779b9;  /* the golden ratio; an arbitrary value */
    for ( i = 0, j = 0; i < DWORD_BYTE_PER_KEY; i += 4 )
      *(size_t*)(key + i) = 0x9e3779b9;
    
    for ( i = 0, j = 0; i < arr_size; i += 12 )
    {
      if ( j < DWORD_BYTE_PER_KEY)
        c = *(size_t*)(key + j);
      else
        c = rest;
                
      /*---------------------------------------- handle most of the key */
      if (arr_size - i >= 12) 
      {
        a += *(size_t*)(st_arr + i);
        b += *(size_t*)(st_arr + i + 4);
        c += *(size_t*)(st_arr + i + 8);
        jmix(a,b,c);
      }
      else
      {
        /*------------------------------------- handle the last 11 bytes */
        c += arr_size;
        switch (arr_size - i)              /* all the case statements fall through */
        {
        case 11: c+=((size_t)st_arr[i + 10]<<24);
        case 10: c+=((size_t)st_arr[i + 9]<<16);
        case 9 : c+=((size_t)st_arr[i + 8]<<8);
          /* the first byte of c is reserved for the length */
        case 8 : b+=((size_t)st_arr[i + 7]<<24);
        case 7 : b+=((size_t)st_arr[i + 6]<<16);
        case 6 : b+=((size_t)st_arr[i + 5]<<8);
        case 5 : b+=(size_t)st_arr[i + 4];
        case 4 : a+=((size_t)st_arr[i + 3]<<24);
        case 3 : a+=((size_t)st_arr[i + 2]<<16);
        case 2 : a+=((size_t)st_arr[i + 1]<<8);
        case 1 : a+=(size_t)st_arr[i];
          /* case 0: nothing left to add */
        }
        jmix(a,b,c);
      }
      
      /* store result */
      if ( j < DWORD_BYTE_PER_KEY )
      {
        *(size_t*)(key + j) = c;
        j += sizeof( size_t );
        if ( j >= DWORD_BYTE_PER_KEY )
          j = 0;
      }
      else
      {
        rest = c;
        j = 0;
      }
    }
    
    j = HASH_KEY_BYTE_SIZE & 0x3;
    switch ( j )
    {
    case 3:
      key[HASH_KEY_BYTE_SIZE - 3] = (uchar)((rest >> 16) & 0xFF);
    case 2:
      key[HASH_KEY_BYTE_SIZE - 2] = (uchar)((rest >> 8) & 0xFF);
    case 1:
      key[HASH_KEY_BYTE_SIZE - 1] = (uchar)(rest & 0xFF);
    }
}

void
StateHash::pearson_hash(uchar *st_arr, int arr_size, uchar *key)
{
    int i, j;
    unsigned short sum = 0;
    //uchar rest = 0;
    key[0] = (arr_size /*+ HASH_KEY_BYTE_SIZE*/) & 0xFF;
    for ( i = 0; i < HASH_KEY_BYTE_SIZE; i++ )
      key[i] = PearsonX[(key[i - 1] + arr_size) & 0xFF];
    
    for ( i = 0; i < arr_size; i++, j++ )
    {
      if ( j >= HASH_KEY_BYTE_SIZE )
        j = 0;
        
      sum = (unsigned short)key[j] + (unsigned short)st_arr[i] /*+ (unsigned short)rest*/;
      key[j] = PearsonX[sum & 0xFF];
        //rest = (uchar)(sum >> 8);
    }
}



uchar StateHash::pack_state_hash_key( StateBase* st, uchar **key )
{
  uchar *comp_st = 0;
  int state_size = st->get_state_byte_array( &comp_st );
  
  if ( max_st_arr_size < state_size )
    max_st_arr_size = state_size;
    
  *key = new uchar[ HASH_KEY_BYTE_SIZE ];
  
  memset( *key, 0, HASH_KEY_BYTE_SIZE * sizeof(uchar) );
  
  if ( state_size > HASH_KEY_BYTE_SIZE )  
    switch (hash_function)
    {
    case PEARSON_FUNC:
      pearson_hash( comp_st, state_size, *key );
      break;
    default:
      jenkins_hash( comp_st, state_size, *key );
    }
  else
    memcpy( *key, comp_st, state_size );
    
  delete[] comp_st;
  
  return (state_size < HASH_KEY_BYTE_SIZE)? state_size: HASH_KEY_BYTE_SIZE;
}


/*
	PDS: Its odd that m_arr can sometimes be 0.  Not all of the
	functions handle that case.  Why permit it?  0 length arrays
	work fine.
*/
BoolArr::BoolArr()
{
	m_bit_count = 0;
	m_arr = 0;
}

BoolArr::BoolArr(BoolArr *arr)
{
	int n;
	
	m_bit_count = arr->m_bit_count;
	n = (m_bit_count == 0) ? 0 : (m_bit_count-1)/CHAR_BIT+1;
	if(n > 0){
		m_arr = new uchar[n];
		memcpy(m_arr, arr->m_arr, n);
	}
	else
		m_arr = 0;
}

BoolArr::BoolArr(int size)
{
	int n;

	m_bit_count = size;
	n = (m_bit_count == 0) ? 0 : (m_bit_count-1)/CHAR_BIT+1;
	if(n > 0){
		m_arr = new uchar[n];
		memset(m_arr, 0, n);
	}
	else
		m_arr = 0;
}

BoolArr::~BoolArr()
{
	delete[] m_arr;
	m_arr = 0;
}

/*
	PDS: Itsn't it odd that set_size discards the old data?
*/
void
BoolArr::set_size(int size)
{
	int n;

	m_bit_count = size;
	if(m_arr != 0)
		delete[] m_arr;
	n = (size == 0) ? 0 : (size-1)/CHAR_BIT+1;
	if(n > 0){
		m_arr = new uchar[n];
		memset(m_arr, 0, n);
	}
	else
		m_arr = 0;
}

int
BoolArr::get_size()
{
	return m_bit_count;
}

void
BoolArr::set_var(int ind, bool val)
{
	uchar* byte;
	uchar bit;
	
	assert(0 <= ind && ind < m_bit_count);	// PDS: Used to be a noop.

	byte = m_arr + ind/CHAR_BIT;
	bit = (uchar)0x80 >> (ind % CHAR_BIT);
	if(val)
		*byte |= bit;
	else
		*byte &= ~bit;
}

bool
BoolArr::is_var_enabled(int ind)
{
	uchar* byte;
	uchar bit;
	
	assert(0 <= ind && ind < m_bit_count);	// PDS: Used to be a noop.
	
	byte = m_arr + ind/CHAR_BIT;
	bit = (uchar)0x80 >> (ind % CHAR_BIT);
	return (*byte)&bit;
}

bool
BoolArr::operator!=(BoolArr &other)
{
	int n, r;
	
	if(m_bit_count != other.m_bit_count)
		return true;
	n = (m_bit_count == 0) ? 0 : (m_bit_count-1)/CHAR_BIT+1;
	r = memcmp(m_arr, other.m_arr, n) != 0;	// PDS: Used to be ==.
	return r;
}

BoolArr&
BoolArr::operator=(const BoolArr &other)
{
	int n;

	if(this != &other){
		m_bit_count = other.m_bit_count;
		if(m_arr != 0)
			delete[] m_arr;
		n = (m_bit_count == 0) ? 0 : (m_bit_count-1)/CHAR_BIT+1;
		if(n > 0){	// PDS: Didn't used to handle n==0 case.
			m_arr = new uchar[n];
			memcpy(m_arr, other.m_arr, n);
		}
		else
			m_arr = 0;
	}
	return *this;
}

void
BoolArr::debug_print()
{
	int n, i;

	n = (m_bit_count == 0) ? 0 : (m_bit_count-1)/CHAR_BIT+1;
	for(i=0; i<n; i++)
		cout << (int)m_arr[i] << ",";
	cout << "\n";
}

/* PDS: No size check for copy_to. */
void
BoolArr::copy_bit_arr(uchar *copy_to)
{
	int n;
	
	n = (m_bit_count == 0) ? 0 : (m_bit_count-1)/CHAR_BIT+1;
	memcpy(copy_to,m_arr,n);
}

const uchar Tracer::QUEUE_CLASS = 1;
const uchar Tracer::AUTOM_CLASS = 2;
const uchar Tracer::STATE_CLASS = 3;
const uchar Tracer::QUEUE_EL_CLASS = 4;
const int Tracer::MAX_ASCII_LINE_LENGTH = 30000;
int Tracer::m_curr_line_len = 0;

const unsigned long Tracer::ASCII_OUTPUT = 0x000001;
const unsigned long Tracer::PRINT_TO_STR = 0x000002;
const unsigned long Tracer::LEFTY_INPUT = 0x000004;
const unsigned long Tracer::GRAPH_ATTR_FORMAT = 0x000008;

const unsigned long Tracer::TRACE_STATE = 0x000010;
const unsigned long Tracer::TRACE_QUEUE = 0x000020;
const unsigned long Tracer::TRACE_AUTOM = 0x000040;
const unsigned long Tracer::TRACE_QUEUE_EL = 0x000080;
const unsigned long Tracer::EXPAND_QUEUE = 0x000100;
const unsigned long Tracer::TRACE_MEM = 0x000200;
const unsigned long Tracer::TRACE_TABLE = 0x000400;
const unsigned long Tracer::ATTR_INIT = 0x000800;
const unsigned long Tracer::TRACE_LOC_VAR = 0x001000;
const unsigned long Tracer::TRACE_GL_VAR = 0x002000;
const unsigned long Tracer::TRACE_VAR = 0x003000;

const unsigned long Tracer::BIN32 = 0x004000;
 
const unsigned long Tracer::ACTION_DEL = 0x010000;
const unsigned long Tracer::ACTION_ADD = 0x020000;
const unsigned long Tracer::ACTION_CLONE = 0x040000;
const unsigned long Tracer::ACTION_VAR = 0x080000;
const unsigned long Tracer::ACTION_LOOK = 0x100000;
const unsigned long Tracer::ACTION_FLAGS = 0xFF0000;

string Tracer::EOL = "\n";
string Tracer::TAB = "\t";
string Tracer::ATTR_START = "{";
string Tracer::ATTR_END = "}";

const unsigned long Tracer::FULL_TRACE = 0x0FBFF0;
unsigned long Tracer::m_tracing_flags = FULL_TRACE;
ostream* Tracer::m_out_stream = 0;
ostringstream Tracer::m_out_sstream("");

NameStorage Tracer::m_name_set;
vector<string> Tracer::m_name_arr(0);
TypeStorage Tracer::m_type_set;
vector<string> Tracer::m_type_arr(0);

void
Tracer::write_1bytes(char b, ostream *s)
{
	s->put(b);
}

void
Tracer::write_2bytes(short b, ostream *s)
{
	/* PDS: Fix byte order. */
	s->write((char*)&b, 2);
}

void
Tracer::write_4bytes(long b, ostream *s)
{
	/* PDS: Fix byte order. */
	s->write((char*)&b, 4);
}

void
Tracer::write_bin_identifier(long id, ostream *s)
{
	if(m_tracing_flags & BIN32)
		write_4bytes(id, s);
	else
		write_2bytes((short)id, s);
}

uchar
Tracer::get_tracer_class_id()
{
	return 0;
}

char
Tracer::add_type(const string &type)
{
	char id;
	TypeStorage::iterator it;

	id = 0;
	if(!type.empty()){
		if((it = m_type_set.find(type)) != m_type_set.end())
			id = it->second;
		else{
			m_type_arr.push_back(type);
			id = (char)m_type_arr.size();
			m_type_set[type] = id;		/* id starts from 1 */
		}
	}
	return id;
}

long
Tracer::add_name(const string &name)
{
	long id;
	NameStorage::iterator it;

	id = 0;
	if(!name.empty()){
		if((it = m_name_set.find(name)) != m_name_set.end())
			id = it->second;
		else{
			m_name_arr.push_back(name);
			id = (long)m_name_arr.size();
			m_name_set[name] = id;	/* id starts from 1 */
		}
	}
	return id;
}

char
Tracer::get_type_id(const string& type)
{
	char id;

	id = 0;
	if(type == ATTR_START)
		id = -1;
	else if(type == ATTR_END)
		id = -2;
	else 
		id = add_type(type);
	return id;
}

long
Tracer::get_name_id(const string& name)
{
	long id;
	
	id = add_name(name);
	return id;
}

void
Tracer::clear_names_storage()
{
	m_name_set.clear();
	m_name_arr.clear();
	m_name_arr.resize(0);
	m_type_set.clear();
	m_type_arr.clear();
	m_type_arr.resize(0);
}

long
Tracer::calc_names_storage_size()
{
	long res;
	unsigned int i;

	res = 0;
	for(i = 0; i < m_name_arr.size(); i++)
		res += 2 * (m_name_arr[i].length() + sizeof(string)) + sizeof(short);
	for(i = 0; i < m_type_arr.size(); i++)
		res += 2 * (m_type_arr[i].length() + sizeof(string)) + sizeof(short);
	return res;
}

long
Tracer::name_number()
{
	return m_name_arr.size();
}

void
Tracer::write_names_storage(ostream *s)
{
	unsigned int i;
	
	for(i=0; i < m_name_arr.size(); i++){
		write_bin_identifier(m_name_arr[i].length(), s);
		(*s) << m_name_arr[i];
	}
	write_bin_identifier(-3, s);	/* internal separator 2 */
	for(i=0; i < m_type_arr.size(); i++){
		write_bin_identifier(m_type_arr[i].length(), s);
		(*s) << m_type_arr[i];
	}
}

void
Tracer::print(const string &info_prefix, long info, const string &info_suffix)
{
	if(m_tracing_flags & ASCII_OUTPUT){
		if((m_tracing_flags & PRINT_TO_STR))
			m_out_sstream << info_prefix << info << info_suffix;
		else{
			if(m_out_stream == 0)
				cout << info_prefix << info << info_suffix;
			else
				(*m_out_stream) << info_prefix << info << info_suffix;
		}
	}
}

void
Tracer::print(const string &info_prefix, const string &info, const string &info_suffix)
{
	if(m_tracing_flags & ASCII_OUTPUT){
		if((m_tracing_flags & PRINT_TO_STR))
			m_out_sstream << info_prefix << info << info_suffix;
		else{
			if(m_out_stream == 0)
				cout << info_prefix << info << info_suffix;
			else
				(*m_out_stream) << info_prefix << info << info_suffix;
		}
	}
}

void
Tracer::print(long info, const string &info_suffix)
{
	print("", info, info_suffix);
}

void
Tracer::print(const string &info)
{
	print("",info,"");
}

void
Tracer::finish_ASCII_block(bool is_end)
{
	if(!is_end)
		print("\\");
	print("\n");
	m_curr_line_len = 0;
}

void
Tracer::print_attr(const string& type, const string& name, long value)
{
	long id;
	char tyid;

	if(m_tracing_flags & ASCII_OUTPUT){
		if(type == ATTR_START){
			if(value >= 0){
				print(" ", value, ":");
				m_curr_line_len += 4;
			}
			print(" ", name, type);
			m_curr_line_len += name.length() + type.length();
		}
		else if(type == ATTR_END){
			print(type, EOL);
			m_curr_line_len += type.length();
		}
		else{
			print(" " + type + ": " + name + "=", value, ";" + EOL);
			m_curr_line_len += name.length() + type.length() + 7;
		}
		if(m_curr_line_len > MAX_ASCII_LINE_LENGTH)
			finish_ASCII_block(false);
	}
	else{
		id = add_name(name);
		write_bin_identifier(id, m_out_stream);
		if(type == ATTR_START)
			write_1bytes(0xFF, m_out_stream);
		else if(type == ATTR_END)
			write_1bytes(0xFE, m_out_stream);
		else{
			tyid = add_type(type);
			write_1bytes(tyid, m_out_stream);
			if(type == "bool" || type == "byte" || type == "bit")
				write_1bytes((unsigned char)value, m_out_stream);
			else if(type == "short" || type == "chan")
				write_2bytes((short)value, m_out_stream);
			else
				write_4bytes((long)value, m_out_stream);
		}
	}
}

void
Tracer::print_attr_ind(const string& type, long name, long value)
{
	char tyid;

	if(m_tracing_flags & ASCII_OUTPUT){
		if(type == ATTR_START){
			if(value >= 0){
				print(" ", value, ":");
				m_curr_line_len += 4;
			}
			print(" ", m_name_arr[name - 1], type);
			m_curr_line_len += m_name_arr[name - 1].length() + type.length();
		}
		else if(type == ATTR_END){
			print(type, EOL);
			m_curr_line_len += type.length();
		}
		else{
			print(" " + type + ": " + m_name_arr[name - 1] + "=", value, ";" + EOL);
			m_curr_line_len += m_name_arr[name - 1].length() + type.length() + 7;
		}
		if(m_curr_line_len > MAX_ASCII_LINE_LENGTH)
			finish_ASCII_block(false);
	}
	else{
		write_bin_identifier(name, m_out_stream);
		if(type == ATTR_START)
			write_1bytes(0xFF, m_out_stream);
		else if(type == ATTR_END)
			write_1bytes(0xFE, m_out_stream);
		else{
			tyid = add_type(type);
			write_1bytes(tyid, m_out_stream);
			if(type == "bool" || type == "byte" || type == "bit")
				write_1bytes((unsigned char)value, m_out_stream);
			else if(type == "short" || type == "chan")
				write_2bytes((short)value, m_out_stream);
			else
				write_4bytes((long)value, m_out_stream);
		}
	}
}

void
Tracer::trace(unsigned long trace_action, long extra_info, const string &line_prefix)
{
}

void
Tracer::put_error(const string &s)
{
	cerr << s;
	exit(0);	/* PDS: Zero status? */
}

unsigned long
Tracer::get_flags()
{
	return m_tracing_flags;
}

void
Tracer::set_flags(unsigned long tracing_flags)
{
	if(tracing_flags != 0xFFFFFFFF){
		if((tracing_flags & GRAPH_ATTR_FORMAT)){
			tracing_flags |= TRACE_STATE | TRACE_AUTOM
				| TRACE_VAR | TRACE_QUEUE | EXPAND_QUEUE;
			TAB = "";
		}
		else if((tracing_flags & LEFTY_INPUT)){
			tracing_flags |= TRACE_STATE | TRACE_AUTOM
				| TRACE_VAR | PRINT_TO_STR;
			TAB = " ";
		}
		else
			TAB = "\t";

		m_tracing_flags = tracing_flags;

		if((tracing_flags & PRINT_TO_STR))
			m_out_sstream.str("");

		if((tracing_flags & GRAPH_ATTR_FORMAT))
			EOL = "";
		else if((tracing_flags & PRINT_TO_STR))
			EOL = "\\n";
		else
			EOL = "\n";
	}
}

ostream*
Tracer::get_curr_stream()
{
	return m_out_stream;
}

void
Tracer::init_tracing(ostream *stream, unsigned long flags)
{
	set_flags(flags);
	m_out_stream = stream;
}

string
Tracer::get_output_str()
{
	string str;
	
	str = m_out_sstream.str();
	m_out_sstream.str("");
	return str;
}

void
Tracer::trace(unsigned long trace_action, Tracer* tracer, long l_info, const string &line_prefix, const string &extra_info)
{
	trace_action &= m_tracing_flags;
	if(trace_action != 0){
		if(tracer == 0){
			print(line_prefix, l_info, extra_info);
		}
		else if((trace_action & TRACE_STATE)
		&& tracer->get_tracer_class_id() == STATE_CLASS){
			print(line_prefix, extra_info);
			tracer->trace(trace_action, l_info, line_prefix);
		}
		else if(((trace_action & TRACE_QUEUE) || (trace_action & TRACE_QUEUE_EL))
		&& tracer->get_tracer_class_id() == QUEUE_CLASS){
			print(line_prefix, extra_info);
			tracer->trace(trace_action, l_info, line_prefix);
		}
		else if(((trace_action & TRACE_AUTOM) || (trace_action & TRACE_TABLE))
		&& tracer->get_tracer_class_id() == AUTOM_CLASS){
			print(line_prefix, extra_info);
			tracer->trace(trace_action, l_info, line_prefix);
		}
		else if((trace_action & TRACE_QUEUE_EL)
		&& tracer->get_tracer_class_id() == QUEUE_EL_CLASS){
			print(line_prefix, extra_info);
			tracer->trace(trace_action, l_info, line_prefix);
		}
	}
}

void
Tracer::trace(Tracer* tracer, long l_info, const string &line_prefix, const string &extra_info)
{
	trace(m_tracing_flags, tracer, l_info, line_prefix, extra_info);
}

QueueEl::~QueueEl()
{
}

uchar
QueueEl::get_tracer_class_id()
{
	return QUEUE_EL_CLASS;
}

void
QueueEl::trace_fld(unsigned long trace_action, long extra_info, const string &line_prefix)
{
}

void 
QueueEl::trace(unsigned long trace_action, long extra_info, const string &line_prefix)
{
	if((trace_action & GRAPH_ATTR_FORMAT)
	&& (m_tracing_flags & EXPAND_QUEUE))
		trace_fld(trace_action, 0, line_prefix);
	else if((trace_action & LEFTY_INPUT) == 0){
		if((trace_action & ACTION_FLAGS) != 0){
			print(line_prefix);
			if((trace_action & ACTION_DEL))
				print("Delete QueueEl ");
			else if((trace_action & ACTION_ADD))
				print("Add QueueEl ");
			else if((trace_action & ACTION_LOOK))
				print("Look QueueEl ");
			if(extra_info < 0)
				print("-> not found" + EOL);
			else{
				print("id: ", extra_info,  " ");
				print("(QueueEl", get_type(), ", ");
				print("fld_count: ", fld_number(), ")" + EOL);
			}
		}
		if((m_tracing_flags & EXPAND_QUEUE))
			trace_fld(trace_action, 0, line_prefix);
		if((m_tracing_flags & TRACE_MEM))
			print(line_prefix + "QueueEl size: ", sizeof(*this), EOL);
	}
}

Queue::Queue(unsigned int max_el, QueueTypeCarrier *q_type_el)
{
	m_type_carrier = q_type_el;
	m_rvz = false;
	if(max_el == 0){
		m_max_length = 1;
		m_rvz = true;
	}
	else
		m_max_length = max_el;
	m_is_enable = true;
	m_owner = -1;
	m_items.reserve(m_max_length);
	m_items.resize(0, 0);
}

Queue::~Queue()
{
	int i;

	for(i=0; i < m_items.size(); i++)
		delete m_items[i];
	delete m_type_carrier;
}

Clonable*
Queue::clone( )
{
	Clonable *q_clone;
	QueueTypeCarrier *q_t;
	Queue *the_clone;
	QueueEl *q_el;
	int i;
	
	q_clone = m_type_carrier->clone();
	q_t = dynamic_cast<QueueTypeCarrier*>(q_clone);
	the_clone = new Queue((is_rvz() ? 0 : m_max_length), q_t);
	q_el = 0;
	for(i=0; i < m_items.size(); i++)
		if(m_items[i] != 0){
			q_clone = m_items[i]->clone();
			q_el = dynamic_cast<QueueEl*>(q_clone);
			the_clone->send(q_el);
		}
	the_clone->m_owner = m_owner;
	return the_clone;
}

QueueEl*
Queue::create_el()
{
	QueueEl* r;
	
	r = m_type_carrier->new_element();
	return r;
}

uchar
Queue::get_tracer_class_id()
{
	return QUEUE_CLASS;
}

void
Queue::trace(unsigned long trace_action, long extra_info, const string &line_prefix)
{
	bool el_created;
	QueueEl *el;
	int i;
	long size;
	
	if((trace_action & TRACE_QUEUE)){
		if((trace_action & GRAPH_ATTR_FORMAT))
			print_attr(ATTR_START, "channel", extra_info);
		else{
			if((trace_action & LEFTY_INPUT) == 0){
				if((trace_action & ACTION_VAR) == 0){
					print(line_prefix);
					if((trace_action & ACTION_DEL))
						print("Delete Queue ");
					else if((trace_action & ACTION_ADD))
						print("Add Queue ");
				}
			}
			print("id: ", extra_info,  " ");
		}
		if((trace_action & LEFTY_INPUT)
		&& (trace_action & GRAPH_ATTR_FORMAT) == 0){
			print("(Q", m_type_carrier->get_type(), ", ");
			print("len: ", m_items.size(), ")" + EOL);
		}
		else{
			if((trace_action & GRAPH_ATTR_FORMAT) == 0){
				print("(Queue", m_type_carrier->get_type(), ", ");
				print("max_len: ", ((m_rvz)? 0: m_max_length), ", ");
				print("len: ", m_items.size(), ")" + EOL);
				print(line_prefix, "{" + EOL);
			}
			el_created = false;
			if((trace_action & ATTR_INIT)
			&& (trace_action & EXPAND_QUEUE)
			&& m_items.size() == 0){
				el = create_el();
				m_items.push_back(el);
				el_created = true;
			}
			if((trace_action & GRAPH_ATTR_FORMAT))
				print_attr("int", "max_size", ((m_rvz)? 0: m_max_length));
			if((trace_action & EXPAND_QUEUE)
			&& m_items.size() > 0)
				for(i=0; i < m_items.size(); i++){
					if((trace_action & GRAPH_ATTR_FORMAT))
						print_attr(ATTR_START, "", i);
					m_items[i]->trace(trace_action, 0, line_prefix + TAB);
					if((trace_action & GRAPH_ATTR_FORMAT))
						print_attr(ATTR_END);
				}
			print_attr(ATTR_END);
			if(el_created){
				delete m_items[m_items.size() - 1];
				m_items.pop_back();
			}
			if((m_tracing_flags & TRACE_MEM)
			&& (trace_action & GRAPH_ATTR_FORMAT) == 0){
				size = sizeof(Queue) + sizeof(*m_type_carrier);
				for(i=0; i < m_items.size(); i++)
					if(m_items[i] != 0)
						size += sizeof(*(m_items[i]));
				size += m_items.size() * sizeof(QueueEl*);
				print(line_prefix + "Queue size: ", size, EOL);
			}
		}
	}
}

bool
Queue::is_empty()
{
	return m_items.empty();
}

bool
Queue::is_full()
{
	return m_items.size() == m_max_length;
}

int
Queue::get_len()
{
	return m_items.size();
}

void
Queue::set_enable(bool enable)
{
	m_is_enable = enable;
}

bool
Queue::get_enable()
{
	return m_is_enable;
}

bool
Queue::is_rvz()
{
	return m_rvz;
}

int
Queue::get_type()
{
	return m_type_carrier->get_type();
}

bool
Queue::send(QueueEl* el)
{
	if(el == 0 || el->get_type() != get_type())
		return false;
	m_items.push_back(el);
	return true;
}

bool
Queue::look(int ind, QueueEl** el)
{
	int s;
	
	s = m_items.size();
	if(ind < 0 || ind >= s){	/* PDS: was if(m_items.size() == 0) */
		*el = 0;
		return false;
	}
	*el = m_items[ind];
	return true;
}

bool
Queue::look(QueueEl** el)
{
	int s;
	
	s = m_items.size();
	if(s == 0)
		return false;
	*el = m_items[s-1];
	return true;
}

bool
Queue::receive(QueueEl **el)
{
	bool rcvd;
	
	rcvd = look(el);
	if(rcvd)
		m_items.pop_back();
	return rcvd;
}

int
Queue::has(string fld_mask, QueueEl *el)
{
	int i;
	
	for(i=m_items.size()-1; i>=0; i--)
		if(m_items[i]->equal(fld_mask,el))
			return i;
	return -1;
}

void
Queue::del(int ind)
{
	int n, pos, i;
	
	n = m_items.size();
	if(ind >= 0 && ind < n){
		pos = n-1-ind;
		if(m_items[pos])
			delete m_items[pos];
		for(i=pos; i<n; i++)
			m_items[i] = m_items[i+1];
		m_items.pop_back();
	}
}

long
Queue::calc_size()
{
	long size;
	int i;
	
	size = sizeof(Queue) + sizeof(*m_type_carrier);
	for(i=0; i < m_items.size(); i++)
		if(m_items[i] != 0)
			size += sizeof(*m_items[i]);
	size += m_items.size() * sizeof(QueueEl*);
	return size;
}

int StateBase::state_in_memory = 0;
int StateBase::max_state_in_memory = 0;
unsigned int StateBase::max_state_size = 0;

StateBase::StateBase()
{
	m_flags = 0;
	m_ltl = 0;
	m_autom_list.reserve(1);
	m_autom_list.resize(0,0);
}

StateBase::~StateBase()
{
	delete_automs();
}

void
StateBase::delete_automs()
{
	int i;

	for(i=0; i < m_autom_list.size(); i++)
		if(m_autom_list[i] != 0){
			delete m_autom_list[i];
			m_autom_list[i] = 0;
		}
	delete m_ltl;
	m_ltl = 0;
}

void
StateBase::copy_to(StateBase *st)
{
	CAutomBase *a;
	Clonable *t_clone;
	int i;

	a = 0;
	t_clone = 0;
	for(i=0; i < m_autom_list.size(); i++){
		if(m_autom_list[i] == 0)
			st->m_autom_list[i] = 0;
		else{
			t_clone = m_autom_list[i]->clone();
			a = dynamic_cast<CAutomBase*>(t_clone);
			st->m_autom_list[i] = a;
			a->m_owner = st;
		}
	}
	st->m_flags = m_flags;
	if(m_ltl != 0){
		t_clone = m_ltl->clone();
		st->m_ltl = dynamic_cast<CAutomBase*>(t_clone);
	}
}

uchar
StateBase::get_tracer_class_id()
{
	return STATE_CLASS;
}

void
StateBase::trace(unsigned long trace_action, long extra_info, const string &line_prefix)
{
	int i;

	if((trace_action & GRAPH_ATTR_FORMAT)){
		if((trace_action & TRACE_AUTOM)
		&& m_ltl != 0)
			m_ltl->trace(trace_action, -1, line_prefix);
		if((trace_action & TRACE_AUTOM)){
			print_attr(ATTR_START, "Processes");
			for(i=0; i < m_autom_list.size(); i++){
				if(m_autom_list[i] != 0)
					m_autom_list[i]->trace(trace_action, i, line_prefix);
			}
			print_attr(ATTR_END, "Processes");
		}
	}
	else{
		if((trace_action & LEFTY_INPUT) == 0){
			print(line_prefix);
			if((trace_action & ACTION_CLONE))
				print("Clone ");
			print("State:" + EOL);
		}
		if((trace_action & TRACE_AUTOM)){
			if(m_ltl == 0)
				print(line_prefix + TAB + "::never:: = 0" + EOL);
			else
				m_ltl->trace(trace_action, -1, line_prefix + TAB);
		}
		if((trace_action & TRACE_AUTOM)){
			if((trace_action & LEFTY_INPUT) == 0)
				print(line_prefix, TAB + "Processes:" + EOL);
			for(i=0; i < m_autom_list.size(); i++)
				if(m_autom_list[i] == 0){
					if((trace_action & LEFTY_INPUT) == 0)
						print(line_prefix + TAB + "  " + "id: ", i, " = (CAutomBase)0" + EOL);
				}
				else
					m_autom_list[i]->trace(trace_action, i, line_prefix + TAB +"  ");
		}
	}
}

unsigned int
StateBase::calc_size()
{
	unsigned int size, i;
	
	size = 0;
	for(i=0; i < m_autom_list.size(); i++)
		if(m_autom_list[i] != 0)
			size += sizeof(*(m_autom_list[i]));
	if(m_ltl != 0)
		size += sizeof(m_ltl);
	size += m_autom_list.size() * sizeof(CAutomBase*);
	return size;
}

enum {
	Found	=1<<0,
	Timeout	=1<<1,
	Progress	=1<<2,
	Stutter	=1<<3,
};

static uchar
setf(int f, bool v, uchar acc)
{
	if(v)
		acc |= f;
	else
		acc &= ~f;
	return acc;
}

bool
StateBase::is_stutter()
{
	return m_flags & Stutter;
}

void
StateBase::set_stutter(bool stutter)
{
	m_flags = setf(Stutter,stutter,m_flags);
}

bool
StateBase::is_progress()
{
	return m_flags & Progress;
}

void
StateBase::set_progress(bool progress)
{
	m_flags = setf(Progress,progress,m_flags);
}

bool
StateBase::is_timeout()
{
	return m_flags & Timeout;
}

void
StateBase::set_timeout(bool timeout)
{
	m_flags = setf(Timeout,timeout,m_flags);
}

bool
StateBase::is_case_found()
{
	return m_flags & Found;
}

void
StateBase::set_is_case_found(bool found)
{
	m_flags = setf(Found,found,m_flags);
}

int
StateBase::get_boq()
{
	return -1;
}

void
StateBase::set_boq(int boq)
{
}

CAutomBase*
StateBase::get_never_claim()
{
	return m_ltl;
}

int
StateBase::add_proc(CAutomBase *autom)
{
	int index;
	
	if(autom == 0)
		index = -1;
	else{
		m_autom_list.push_back(autom);
		index = m_autom_list.size() - 1;
		autom->set_pid(index);
		autom->m_owner = this;
	}
	return index;
}

CAutomBase*
StateBase::get_proc(int a_num)
{
	if(a_num >= m_autom_list.size() || a_num < 0)
		return 0;
	return m_autom_list[a_num];
}

int
StateBase::proc_count()
{
	int i, r;

	for(i=m_autom_list.size() - 1; i >= 0; i--){
		if(m_autom_list[i] != 0){
			if(m_autom_list[i]->get_curr_state() != 0)
				break;
			delete m_autom_list[i];
			m_autom_list.pop_back();
		}
	}
	r = m_autom_list.size();
	return r;
}

bool
StateBase::get_enabled_flag(int pid)
{
	CAutomBase *proc;
	int trans_id, state_id;
	
	proc = get_proc(pid);
	if(proc != 0){
		state_id = proc->get_curr_state();
		trans_id = proc->get_first_trans_id(state_id);
		for(; trans_id != 0; trans_id = proc->get_next_trans_id(trans_id))
			if(proc->is_trans_enable(this, trans_id))
				return true;
	}
	return false;
}

bool
StateBase::get_progress_flag()
{
	int pid, state_id;
	CAutomBase *proc;

	for(pid=0; pid < proc_count(); pid++){
		proc = get_proc(pid);
		if(proc != 0){
			state_id = proc->get_curr_state();
			if(proc->is_state_in_progress(state_id))
				return false;
		}
	}
	return true;
}

int
StateBase::get_curr_state(int pid)
{
	CAutomBase *proc;
	int state_id;
	
	proc = get_proc(pid);
	if(proc){
		state_id = proc->get_curr_state();
		return state_id;
	}
	return 0;
}

Trans::Trans()
{
}

Trans::Trans(Trans *a)
{
	atom = a->atom;
	st = a->st;
	this_st = a->this_st;
	tp = "";
	tp += a->tp;
	t_id  = a->t_id;
	forw  = a->forw;
	nxt = 0;
}

const uchar CAutomBase::ATOM = 2;
const uchar CAutomBase::L_ATOM = 4;
const uchar CAutomBase::ND_ATOM = 16;
const uchar CAutomBase::D_ATOM = 64;

CAutomBase::CAutomBase()
{
	_pid = 0;
	m_state_number = 0;
	m_curr_state = 0;
	m_flags = 0;
}

bool
CAutomBase::set_state_number(int len)
{
	if(len <= 0)
		return false;
	m_state_number = len;
	return true;
}

CAutomBase::~CAutomBase()
{
}

uchar
CAutomBase::get_tracer_class_id()
{
	return AUTOM_CLASS;
}

void
CAutomBase::trace_autom_base(unsigned long trace_action, long extra_info, const string &name, const string &line_prefix)
{
	if((trace_action & GRAPH_ATTR_FORMAT)){
		if(extra_info >= 0)
			print_attr(ATTR_START, name, _pid);
		else
			print_attr(ATTR_START, name, -1);
	}
	else{
		if((trace_action & LEFTY_INPUT) == 0){
			print(line_prefix);
			if((trace_action & ACTION_ADD))
				print("Add Automaton ");
			else if((trace_action & ACTION_DEL))
				print("Delete Automaton ");
		}
		if(extra_info >= 0)
			print("id: ", _pid, " ");
		print("(" + name +  ") st:", m_curr_state,  EOL);
	}
}

void
CAutomBase::trace_autom_end(unsigned long trace_action, const string &name, const string &line_prefix)
{
	if((trace_action & GRAPH_ATTR_FORMAT))
		print_attr(ATTR_END, name, -1);
	else
		print(line_prefix, ATTR_END, EOL);
}

void
CAutomBase::trace_table(unsigned long trace_action, Trans *** table, int sst_len, bool *accpstate, bool *progstate, long extra_info, const string &line_prefix)
{
	Trans **trans, *T0;
	int i;
	long size;

	if((trace_action & LEFTY_INPUT)
	|| (trace_action & GRAPH_ATTR_FORMAT))
		return;

	trans = *table;
	if(trace_action & ACTION_DEL)
		print(line_prefix + "Delete Trans Table");
	else if(trace_action & ACTION_ADD){
		print(line_prefix + "Add Trans Table" + EOL);
		/* debugging */
		for(i=1; i < sst_len; i++){
			print(line_prefix + TAB);
			if(trans[i] == 0)
				print("id: ", i, " = (Trans)0" + EOL);
			else{
				T0 = trans[i];
				print("{the_st: ", T0->this_st, "; ");
				print("t_id: ", T0->t_id, "; ");
				print("st: ", T0->st, "; ");
				print("atom: ", T0->atom, "; ");
				print("forw: ", T0->forw, "} ");
				for(T0 = T0->nxt; T0; T0 = T0->nxt){
					print("{the_st: ", T0->this_st, "; ");
					print("t_id: ", T0->t_id, "; ");
					print("st: ", T0->st, "; ");
					print("atom: ", T0->atom, "; ");
					print("forw: ", T0->forw, "} ");
				}
				print(EOL);
			}
			print(line_prefix + TAB + "accpstate: ", (int)accpstate[i], ", ");
			print("progstate: ", (int)progstate[i], EOL);
		}
	}
	if(m_tracing_flags & TRACE_MEM){
		size = 0;
		for(i=0; i < sst_len; i++){
			T0 = trans[i];
			size += sizeof(Trans*);
			if(T0 != 0){
				size += sizeof(Trans);
				for(T0 = T0->nxt; T0; T0 = T0->nxt)
					size += sizeof(Trans) + sizeof(Trans*);
			}
		}
		size += sst_len * sizeof(bool);
		print(line_prefix + "Trans Table Size: ", size, EOL);
	}
}

enum{
	TestEnabled	=1<<0,
	Moved	=1<<1,
	FirstSkipTrans	=1<<2,
	InsideAtomic	=1<<3,
	ElsePassed	=1<<4,
};

bool
CAutomBase::is_test_enabled()
{
	return m_flags & TestEnabled;
}

bool
CAutomBase::is_moved()
{
	return m_flags & Moved;
}

void
CAutomBase::set_moved(bool moved)
{
	m_flags = setf(Moved,moved,m_flags);
}

bool
CAutomBase::is_first_skip_trans()
{
	return m_flags & FirstSkipTrans;
}

void
CAutomBase::set_first_skip_trans(bool firstskiptrans)
{
	m_flags = setf(FirstSkipTrans,firstskiptrans,m_flags);
}

bool
CAutomBase::is_inside_atomic()
{
	return m_flags & InsideAtomic;
}

void
CAutomBase::set_inside_atomic(bool insideatomic)
{
	m_flags = setf(InsideAtomic,insideatomic,m_flags);
}

bool
CAutomBase::is_else_passed()
{
	return m_flags & ElsePassed;
}

void
CAutomBase::set_else_passed(bool elsepassed)
{
	m_flags = setf(ElsePassed,elsepassed,m_flags);
}

void
CAutomBase::set_pid(int pid)
{
	_pid = pid;
}

int
CAutomBase::get_curr_state()
{
	return m_curr_state;
}

int
CAutomBase::get_state_count()
{
	return m_state_number;
}

bool
CAutomBase::set_curr_state(int st)
{
	/* PDS: Expected st >= 0 && st < m_state_number */
	if(st >= 0 || st < m_state_number){
		m_curr_state = st;
		return true;
	}
	return false;
}

bool
CAutomBase::is_enable()
{
	return m_curr_state > 0;
}

bool
CAutomBase::is_stop_found_first()
{
	return m_stop == (uchar)FIRST_STOP;
}

bool
CAutomBase::is_trans_enable(StateBase *state, int tr)
{
	bool r;
	int cur_st, loc_tr;
	
	m_flags = setf(TestEnabled,true,m_flags);
	/* tr range verified here */
	r = new_state(state,tr);
	if(!r && is_first_skip_trans()){
		cur_st = get_curr_state();
		loc_tr = get_first_trans_id(cur_st);
		for(; loc_tr != 0; loc_tr = get_next_trans_id(loc_tr)){
			r = new_state(state,loc_tr);
			if(r)
				break;
		}
	}
	set_first_skip_trans(false);
	m_flags = setf(TestEnabled,false,m_flags);
	return r;
}

void
CAutomBase::del_transition(Trans *t)
{
/* PDS: Isn't del_transition(t) identical to delete t? */
	if(t)
		delete t;
}

void
CAutomBase::imed(bool *accpstate, bool *progstate, int st1, int st2)
{
/* PDS: Assumes st1 and st2 fit the two arrays.  Eliminate? */
	accpstate[st1] |= accpstate[st2];
	progstate[st1] |= progstate[st2];
}

static int
calc_max_forward(Trans ** sst_table, int sst_len)
{
	int i, max_forw;
	Trans *T0;

	max_forw = 0;
	for(i=0; i < sst_len; i++){
		T0 = sst_table[i];
		for(; T0 != 0; T0 = T0->nxt)
			max_forw ++;
	}
	return max_forw;
}

static void
find_reached(Trans **sst_table, bool *reached, int sst_len, int init_st)
{
	Trans *T, *T0;

	if(init_st < sst_len && init_st > 0 && sst_table[init_st] != 0)
		for(T = sst_table[init_st]; !reached[init_st]; T = sst_table[T->st])
		{
			reached[init_st] = true;
			find_reached(sst_table, reached, sst_len, T->st);
			for(T0 = T->nxt; T0 != 0; T0 = T0->nxt)
				find_reached(sst_table, reached, sst_len, T0->st);
		}
}

/*
	Process n, with m states, is=initial state
*/
void CAutomBase::rebuild_table(Trans ***sst_table, bool *accpstate, bool *progstate, int sst_len, int &init_st)
{
	Trans **trans, *T0, *T1, *T2, *T3;
	int i, j, cnt;
	bool *reached, *forwards_table, atomic;

	trans = *sst_table;
	cnt = calc_max_forward(trans, sst_len);
	reached = new bool[sst_len];
	memset(reached, 0, sizeof(bool) * sst_len);
	
	cout << "max = " << cnt << " ";
	forwards_table = new bool[cnt];
	memset(forwards_table, 0, cnt * sizeof(bool));
	for(i=0, cnt=0; i < sst_len; i++){
		T0 = trans[i];
		for(; T0 != 0; T0 = T0->nxt)
			if(!forwards_table[T0->forw] && T0->forw > 2){
				cnt++;
				forwards_table[T0->forw] = true;
			}
	}
	cout << "frwrds = " << cnt << " ";
	delete[] forwards_table;

	do{
		for(i=1, cnt=0; i < sst_len; i++){
			T2 = trans[i];
			T1 = T2 ? T2->nxt : 0;
			for(T0 = T1; T0; T0 = T0->nxt){
				if(T0->st && trans[T0->st] && trans[T0->st]->nxt)
					break;
			}
			if(T0)
				for(T0 = T1; T0; T0 = T0->nxt){
					T3 = trans[T0->st];
					if(!T3->nxt){
						T2->nxt = new Trans(T0);
						T2 = T2->nxt;
						imed(accpstate, progstate, T2->st, T0->st);
					}
					else{
						do{
							T3 = T3->nxt;
							T2->nxt = new Trans(T3);
							T2 = T2->nxt;
							imed(accpstate, progstate, T2->st, T0->st);
						}while(T3->nxt);
						cnt++;
					}
				}
		}
	}while(cnt);

	for(i=1; i < sst_len; i++){
		if(trans[i] != 0 && trans[i]->nxt != 0){	/* optimize */
			T1 = trans[i]->nxt;
			if(trans[T1->st] == 0)
				continue;
			T0 = new Trans(trans[T1->st]);
			//delete trans[i];
			trans[i] = T0;
			T0->this_st = i;
			T0->t_id = i;
			imed(accpstate, progstate, T0->st, T1->st);
			for(T1=T1->nxt; T1; T1 = T1->nxt){
				if(trans[T1->st] == 0)
					continue;
				if(T1->tp == "ATOMIC"){
					T0 = T0->nxt;
					continue;
				}
				T0->nxt = new Trans(trans[T1->st]);
				T0 = T0->nxt;
				T0->this_st = i;
				T0->t_id = i;
				imed(accpstate, progstate, T0->st, T1->st);
			}
		}
	}

	find_reached(trans, reached, sst_len, init_st);

	/* remove unreached */
	for(i=1; i < sst_len; i++){
		if(!reached[i] && trans[i] != 0){
			for(T0=trans[i]->nxt; T0 != 0;){
				T1 = T0;
				T0 = T1->nxt;
				delete T0;
			}
			delete trans[i];
			trans[i] = 0;
		}
	}

	/* finish atomic */
	for(i=1; i < sst_len; i++){
		T0 = trans[i];
		if(T0 != 0){
			atomic = false;
			for(T1=T0; T1 != 0 && !atomic; T1 = T1->nxt)
				atomic = (T1->atom & ATOM) != 0;
			if(atomic && T0->nxt != 0)
				for(T1=T0; T1 != 0; T1 = T1->nxt){
					T1->atom &= ~ATOM;
					T1->atom |= ND_ATOM;
				}
		}
	}

	/* remove skip/goto */
	for(j=1; j < sst_len; j++){
		T0 = trans[j];
		if(T0 != 0
		&& T0->forw == 1
		&& T0->nxt == 0
		&& T0->this_st != T0->st
		&& !(accpstate[T0->this_st])
		&& !(progstate[T0->this_st])){
			for(i=1; i < sst_len; i++)
				if(trans[i] != 0 && T0->this_st != i)
					for(T1=trans[i]; T1 != 0; T1 = T1->nxt)
						if(T1->st == T0->this_st)
							T1->st = T0->st;
			if(T0->this_st == init_st)
				init_st = T0->st;
			imed(accpstate, progstate, T0->st, T0->this_st);
			trans[j] = 0;
			delete T0;
		}
	}

	delete[] reached;

	cnt = calc_max_forward(trans, sst_len);
	cout << " max = " << cnt;
	forwards_table = new bool[cnt];
	memset(forwards_table, 0, cnt);

	for(i=0, cnt = 0; i < sst_len; i++){
		T0 = trans[i];
		for(; T0 != 0; T0 = T0->nxt)
			if(!forwards_table[T0->forw]  && T0->forw > 2){
				cnt++;
				forwards_table[T0->forw] = true;
			}
	}

	cout << "frwds = " << cnt << "\n";
	delete[] forwards_table;
}

void
CAutomBase::del_table(Trans ***sst_table, int &table_ref, int &table_length)
{
	Trans **table;
	int i;

	if(table_ref == 0)
		return;
	if(table_ref == 1){
		if(sst_table != 0){
			Tracer::trace(TRACE_TABLE | ACTION_DEL, this, _pid, "", EOL);

			table = *sst_table;
			if(table != 0){
				/* m_sst_table[0] == 0, always */
				for(i=1; i < table_length; i++){
					del_transition(table[i]);
					table[i] = 0;
				}
				table_length = 0;
				delete[] table;
				*sst_table = 0;
			}
		}
	}
	table_ref--;
}

bool
CAutomBase::start_table_init(Trans ***table, int ref_count, int len)
{
	if(table == 0)
		return false;
	if(ref_count == 0){
		*table = new Trans*[len];
		memset(*table, 0, len * sizeof(Trans*));
	}
	return true;
}

/*
	PDS: A general add_transition lurks behind these three functions.
	It's stupid to use a gazillion arguments.
*/
bool
CAutomBase::add_transition(Trans ***sst_table, int ref_count, int table_len,
	int t_id, short atom, int st, int forw, int backw, string tp, int glob, int tpe0, int tpe1)
{
	Trans *t, **table, *t_next;
	
	if(sst_table == 0 || t_id >= table_len || t_id < 0)
		return false;
	t = 0;
	table = *sst_table;
	if(ref_count == 0){
		t = new Trans;
		if(t == 0)
			return false;
		if(atom & L_ATOM)
			atom |= ATOM;
		t->atom = atom;
		if(!glob)
			t->atom |= 8;  /* no global references */
		t->st = st;
		t->t_id = t_id;
		t->this_st = t_id;
		t->tp = tp;
		t->forw = forw;
		t->nxt = 0;
		if(table[t_id] == 0)
			table[t_id] = t;
		else{
			t_next = table[t_id];
			for(; t_next->nxt != 0; t_next = t_next->nxt)
				;
			t_next->nxt = t;
		}
		/*Calc init state (if not present)*/
		if(m_curr_state == 0)
			m_curr_state = t_id;
	}
	return true;
}

bool
CAutomBase::add_transition(Trans ***sst_table, int ref_count, int table_len,
	int ind, string tp)
{
	Trans *t, **table, *t_next;
	
	if(sst_table == 0 || ind >= table_len || ind < 0)
		return false;
	t = 0;
	table = *sst_table;
	if(ref_count == 0){
		t = new Trans;
		if(t == 0)
			return false;
		t->t_id = ind;
		t->this_st = ind;
		t->tp = tp;
		t->atom = 0;
		t->st = 0;
		t->forw = 0;
		t->nxt = 0;
		if(table[ind] == 0)
			table[ind] = t;
		else{
			t_next = table[ind];
			for(; t_next->nxt != 0; t_next = t_next->nxt)
				;
			t_next->nxt = t;
		}
	}
	return true;
}

bool
CAutomBase::add_transition(Trans ***sst_table, int ref_count, int table_len,
	int t_id, short atom, int st, int forw)
{
	Trans *t, **table, *t_next;

	if(sst_table == 0 || t_id >= table_len || t_id < 0)
		return false;
	t = 0;
	table = *sst_table;
	if(ref_count == 0){
		t = new Trans;
		if(t == 0)
			return false;
		t->atom = atom;
		t->st = st;
		t->t_id = t_id;
		t->this_st = t_id;
		t->forw = forw;
		t->nxt = 0;
		if(table[t_id] == 0)
			table[t_id] = t;
		else{
			t_next = table[t_id];
			for(; t_next->nxt != 0; t_next = t_next->nxt)
				;
			t_next->nxt = t;
		}
	}
	return true;
}

void
CAutomBase::end_table_init(Trans ***table, bool *accpstate, bool *progstate, int &ref_count, int &table_len, int &init_st)
{
	Trans **buff, *t;
	int state_len, offset, i;
	
	buff = 0;
	if(table == 0 || *table == 0){
		table_len = 0;
		ref_count = 0;
	}
	else{
		if(ref_count == 0){
			/* m_curr_state - calculated init state*/
			init_st = m_curr_state;
			if(0)
				rebuild_table(table, accpstate, progstate, table_len, init_st);
			buff = *table;
			state_len = table_len;
			offset = 0;
			t = 0;
			for(i=1; i < state_len; i++){
				if(buff[i] != 0)
					for(t=buff[i]; t->nxt != 0; t = t->nxt)
						offset++;
			}
			if(offset != 0){
				buff = new Trans*[table_len + offset];
				if(buff == 0)
					return;
				memcpy(buff, (*table), table_len * sizeof(Trans*));
				memset(buff + table_len, 0, offset * sizeof(Trans*));
				delete[] (*table);
				*table = buff;
				table_len += offset;
				offset = 0;
				for(i=1; i < state_len; i++){
					t = buff[i];
					if(t != 0)
						for(t = t->nxt; t != 0; t = t->nxt){
							buff[state_len + offset] = t;
							t->t_id = state_len + (offset++);
						}
				}
			}
			Tracer::trace(TRACE_TABLE | ACTION_ADD, this, 0, "", EOL);
		}
		ref_count++;
		/*init_st - real init state*/
		m_curr_state = init_st;
	}
}
